##################
#
# klasa abstrakcyjna dirTree
#
# (c) Marian Mrozek, Krakw 2000
#
# Klasa suy do rekursywnej obrbki katalogw.
# Klasy pochodne implementuj processFile() i ewentualnie
# preProcessDir() oraz postProcessDir()
#
# Generowane klucze:
#   $self->{param}->{baseDir} - Bazowy katalog, ktry podano jako parametr konstruktora
#                               obiektu uruchamiajcego rekursj
#   $self->{currentFileWithPathFromBaseDir} - aktualny plik ze ciek z bazowego katalogu
#   $self->{currentFile} -  aktualny plik bez cieki
#

package dirTree;

use strict;
use utilities;

################################
#
# Konstruktor
#
sub new{
  my ($class,$dir) = @_;
  my $self = {};
  if($dir eq '.' || $dir eq ''){
    $dir=pwd();
  }
  $dir =~ s/\\/\//g;
  if(-d $dir){
    $self->{param}->{baseDir}=dirSlash($dir);
  }else{
    die("dirTree->new: $dir is not a valid directory name" );
  }
  $self->{dir}='';
  $self->{depth}=-1;
  bless($self,$class);
  return $self;
}


################################
#
# Funkcja konstruuje obiekt klasy dirTree przenoszac do niego
# parametry z obiektu rodzicielskiego
#
sub child{
  my ($self) = @_;
  my $child_self = {};
  foreach my $key (keys(%$self)){
    $child_self->{$key}=$self->{$key};
  }
  $child_self->{parent}=$self;
  $child_self->{dir}=dirSlash($self->{currentFileWithPathFromBaseDir});
  $child_self->{param}=$self->{param};
  bless($child_self,ref($self));
  return $child_self;
}


#############################################
#
# preProcessFile
#
sub preProcessFile{
}

#############################################
#
# preProcessDir
#
sub preProcessDir{
}

#############################################
#
# postProcessDir
#
sub postProcessDir{
}

#############################################
#
# processFile
#
sub processFile{
  die("dirTree:: processFile not implemented in abstract class\n");
}

#############################################
#
# recurse
# Argument $doThisFileOnly pozwala
# ograniczy przeszukiwanie katalogu w pierwszym kroku rekursji tylko do podanego
# pliku.
#
sub recurse{
  my ($self,$doThisFileOnly,$ifMatchesThisPattern)=@_;
  my ($numberProcessedFiles)=0;
  ++$self->{depth};
  print "===>BaseDir=$self->{param}->{baseDir} Directory $self->{dir}\n" if $main::opt_d;
  my $dir=("$self->{param}->{baseDir}$self->{dir}" =~ /^(.*)\// ? $1 : '.');
  $dir.='\\' if ($dir =~ /^\w:$/);
  opendir (DIRHANDLE, $dir);
  my @filelist = readdir (DIRHANDLE);
  closedir(DIRHANDLE);
  my $file;
  LOOP:
  foreach $file (@filelist){
    next if($file eq '.' || $file eq "..");
    $self->{currentFile}=$file;
    $self->{currentFileWithPathFromBaseDir}=$self->{dir}.$file;
    my $fullFileName=$self->{param}->{baseDir}.$self->{currentFileWithPathFromBaseDir};
    $self->preProcessFile($fullFileName);
    if(-d $fullFileName){
      print "===> Entering the directory $file\n" if $main::opt_d;
      my $chld=$self->child();
      $self->preProcessDir($fullFileName);
      # just to give the preProcessDir function a chance to control which directories will be recursed:
      next if($self->{currentFile} eq '');
      $self->{childProcessedFiles}=$chld->recurse(); # this information may be needed by postProcessDir
      $numberProcessedFiles+=$self->{childProcessedFiles};
      $self->{numberProcessedFiles}=$numberProcessedFiles;
      $self->postProcessDir($fullFileName);
      print "===> Leaving directory $file \n" if $main::opt_d;
    }else{
      next LOOP if($doThisFileOnly && $file =~ $ifMatchesThisPattern && !($file eq $doThisFileOnly));
      print "===> Processing $file \n" if $main::opt_d;
      $numberProcessedFiles+=$self->processFile($fullFileName);
    }
  }
  --$self->{depth};
  return $numberProcessedFiles;
}

#############################################

1;