##################
#
# klasa StudentRecordBase
# jest to wersja RecordBase
# uwzgldniajca stare sposoby zapisu
# plikw ze studentami
#

package StudentRecordBase;

@ISA=qw(RecordBase);

use FileHandle;
use RecordBase;
use utilities;
use polChar;
use textUtil;
# use egzaminator;

################################
#
# Konstruktor
#
sub new{
  my ($class,$raport) = @_;
  my $self = {};
  bless($self,$class);
# Tego nie wolno robi, bo StudentRecordBase nie moe zawiera adnych extra kluczy
# (bda traktowani jak studenci)
#  $self->{raport}=$egzaminator::biezacyRaport;
  return $self;
}

################################
#
# Funkcja normalizuje klucze i dodatkowo zmienia klucz
#
sub normalizujZmienKlucz{
  my ($klucz)=@_;
  my $staryKlucz=$klucz;

  $klucz='punktyCwiczeniowe' if($klucz eq 'term1_komentarz_pub');
  $klucz='zaliczenie' if($klucz eq 'term1_wart_oc_kolejnosc');

  # normalizacja zmienia pierwsz liter na ma
  # oraz sklejenia przez podkrelenie zmienia
  # na sklejenia przez du liter
  $klucz=normalizujKlucz($klucz);

  $klucz='liczbaPorzadkowa' if($klucz eq 'lp');
  $klucz='indeks' if($klucz eq 'nrAlbumu');
  $klucz='indeks' if($klucz eq 'numerAlbumu');
  $klucz='numerOsoby' if($klucz eq 'numerStudenta');
  $klucz='grupaCwiczeniowa' if($klucz eq 'nazwaGrupy');
  $klucz='grupaCwiczeniowa' if($klucz eq 'nazwaGrupyCwiczeniowej');
  $klucz='grupaWykladowa' if($klucz eq 'nazwaGrWykladowej');
  $klucz='grupaWykladowa' if($klucz eq 'nazwaGrupyWykladowej');
  $klucz='punktyCwiczeniowe' if($klucz eq 'pktc');
  $klucz='punktyEgzaminowe' if($klucz eq 'pktw');
  $klucz='punktyEgzaminowePoprawkowe' if($klucz eq 'pktp');
  $klucz='punktyCwiczeniowe' if($klucz eq 'C-wynikKoncowy' || $klucz eq 'c-wynikKoncowy');
  $klucz='punktyCwiczeniowe' if($klucz eq 'R-wynikKoncowy' || $klucz eq 'r-wynikKoncowy');
  $klucz='punktyCwiczeniowe' if($klucz eq 'term1KomentarzPub');
  $klucz='punktyKonwersatorium' if($klucz eq 'K-wynikKoncowy' || $klucz eq 'k-wynikKoncowy');
  $klucz='rnieob' if($klucz eq 'R-nieob' || $klucz eq 'r-nieob');
  $klucz='knieob' if($klucz eq 'K-nieob' || $klucz eq 'k-nieob');
  $klucz='midTerm1' if($klucz eq 'punktyEgzaminowe-midterm1');
  $klucz='midTerm2' if($klucz eq 'punktyEgzaminowe-midterm2');
  $klucz='midTerm3' if($klucz eq 'punktyEgzaminowe-midterm3');
  print "Zmieniono klucz $staryKlucz na $klucz\n" if($main::opt_v);
  return $klucz;
}

################################
#
# Funkcja zmienia klucz w oparciu o dostarczona tabele konwersji
#
sub zmienKluczKontekstowo{
  my ($klucz,$conversionTableRef)=@_;
  my $staryKlucz=$klucz;
  while (($key,$value) = each %$conversionTableRef) {
    $klucz=$value if($klucz eq $key);
  }
  print "Zmieniono klucz $staryKlucz na $klucz\n" if($main::opt_v);
  return $klucz;
}

################################
#
# Funkcja normalizuje i zmienia klucze w podanej przez
# referencje tablicy
#
sub normalizujZmienKlucze{
  my ($self,$kluczeRef,$conversionTableRef)=@_;
  foreach my $klucz (@$kluczeRef){
    $klucz=normalizujZmienKlucz($klucz);
  }
  if($conversionTableRef){
    foreach my $klucz (@$kluczeRef){
      $klucz=zmienKluczKontekstowo($klucz,$conversionTableRef);
    }
  }
}

################################
#
# Funkcja sprawdza kodowanie i w razie potrzeby konwertuje do
#
sub polConv{
  my ($self,$fileName)=@_;
  my $encoding=plConvertFile2win($fileName);
  my $converted = (($encoding ne '') && ($encoding ne'Win'));
  print "File $filename converted from $encoding encoding to Win\n" if($converted);
}

################################
#
# Funkcja czyta baze z naglowkiem i rekordami
#
sub read{
  my ($self,$fileName,$par,$recordClass)=@_;
  my $fh=new FileHandle;

  # Convert Polish characters if necessary
  $self->polConv($fileName);

  $fh->open("<$fileName") or $egzaminator::biezacyRaport->zakoncz("Failed to open $fileName");
  my @fieldsOrderArray;
  if($fh->getline =~ /\[Nag[l][o]wek\]/){
    $self->readHeading($fh,$par);
    my $fieldsList=eject_blanks($par->{DaneRekordu});
    my $sep=Record::guessSep($fieldsList,'');
    @fieldsOrderArray=split($sep,$fieldsList);
    $self->normalizujZmienKlucze(\@fieldsOrderArray);
  }elsif($line =~ /;\"?pesel\"?;?/  ||  $line =~ /^\"?pesel\"?;?/){
    # Plik w formacie USOS
    $line =~ s/\"//g;
print "USOS format\n";
    @fieldsOrderArray=split(";",$line);
    $self->normalizujZmienKlucze(\@fieldsOrderArray);
    $masterRecord={pesel => 1, indeks => 1, nazwisko => 1, imie => 1};
    $par->{StanListy}='usos';
    $par->{DaneRekordu}='pesel;indeks;nazwisko;imie;';
  }else{
    # Plik w starym formacie bez nagwka, wic cofamy si do pocztku pliku
    $fh->seek(0,0);
    @fieldsOrderArray=('liczbaPorzadkowa','nazwisko','imie','indeks');
  }
  $recordClass->addFields(\@fieldsOrderArray);
  $self->readRecords($fh,$recordClass,\@fieldsOrderArray);
}

################################
#
# Funkcja merguje rekordy
#
sub merge{
  my ($self,$fileName,$par,$recordClass,$masterRecord,$fieldsToRemove,$conversionTableRef)=@_;
# print "Merging with conversionTableRef=$conversionTableRef\n";
  my $fh=new FileHandle;

  # Convert Polish characters if necessary
  $self->polConv($fileName);

  $fh->open("<$fileName") or $egzaminator::biezacyRaport->zakoncz("Failed to open $fileName");
  my @fieldsOrderArray;
  my $line=$fh->getline;
  if($line =~ /\[Nag[l][o]wek\]/){
    $self->readHeading($fh,$par);
    if(!$par->{DaneRekordu}){
      die("Niepoprawny format pliku $fileName: nie znaleziono pola DaneRekordu w nagwku!");
    }
    my $fieldsList=eject_blanks($par->{DaneRekordu});
    my $sep=Record::guessSep($fieldsList,'');
    @fieldsOrderArray=split($sep,$fieldsList);
    $self->normalizujZmienKlucze(\@fieldsOrderArray,$conversionTableRef);
  }elsif($line =~ /;\"?pesel\"?;?/  ||  $line =~ /^\"?pesel\"?;?/){
    # Plik w formacie USOS
    $line =~ s/\"//g;
print "USOS format\n";
    @fieldsOrderArray=split(";",$line);
    $self->normalizujZmienKlucze(\@fieldsOrderArray,$conversionTableRef);
    $masterRecord={
      pesel => 1,
      indeks => 1,
      nazwisko => 1,
      imie => 1,
      punktyCwiczeniowe => 1,
      zaliczenie => 1,
      punktyCwiczeniowe1 => 1,
      zaliczenie1 => 1,
      punktyCwiczeniowe2 => 1,
      zaliczenie2 => 1,
      'zad1' => 1,
      'zad2' => 1,
      'zad3' => 1,
      'zad4' => 1,
      'zad5' => 1
    };
    $par->{StanListy}='usos';
    my $daneRekordu='';
    foreach my $rek (@fieldsOrderArray){
      $daneRekordu.="$rek;";
    }
    $par->{DaneRekordu}=$daneRekordu;
  }else{
    # Plik w starym formacie bez nagwka, wic cofamy si do pocztku pliku
    $fh->seek(0,0);
    @fieldsOrderArray=('liczbaPorzadkowa','nazwisko','imie','indeks');
  }
  $recordClass->addFields(\@fieldsOrderArray);
  my @Fields=@{$recordClass->fields};
  $self->mergeRecords($fh,$recordClass,\@fieldsOrderArray,$masterRecord,$fieldsToRemove);
}

################################
#
# Funkcja czyta baze z naglowkiem i rekordami i dodaje dane do istniejcyh ju rekordw
#
sub mergeData{
  my ($self,$fileName,$parLoc,$recordClass,$silent,$masterRecord)=@_;
  my $fh=new FileHandle;
  $fh->open("<$fileName") or $egzaminator::biezacyRaport->zakoncz("Failed to open $fileName");
  # Jeli $parLoc ma nie okrelon warto, inicjujemy jako referencj do anonimowego hasza
  $parLoc={} if( !defined($parLoc) || $parLoc eq'');
  $self->readHeading($fh,$parLoc);
  my $fieldsList=eject_blanks($parLoc->{DaneRekordu});
  my $sep=Record::guessSep($fieldsList,$sep);
  my @fieldsOrderArray=split($sep,$fieldsList);
  $self->normalizujZmienKlucze(\@fieldsOrderArray);
  #   Na wszelki wypadek mergujemy klucze zawarte w tablicy wyznaczajacej ich porzadek
  $recordClass->addFields(\@fieldsOrderArray);
  $self->mergeRecordsData($fh,$recordClass,\@fieldsOrderArray,$silent,$masterRecord);
}

################################
#
# Funkcja wczytuje studentw z pliku(w) ze studentami
# Jeli jest katalog studenci (lub studenci-edycja)
# czyta z tego katalogu. W przeciwnym razie
# prbuje czyta po staremu
# Zwracany jest napis zawierajcy daneRekordu ostatniego wczytano
# pliku z numerem protokou (plik z dziekanatu)
# lub napis pusty, jeli takiego pliku nie byo
#
#
sub wczytajStudentow{
  my ($self,$par,$raport,$dirxx,$edycja,$masterRecord,$extension)=@_;

  $RecordBase::warnOnFindingNewRecords=0;
  my $oldStyleStudentsFile="$par->{nazwaBazowaTestu}.studenci.csv";
  $extension='csv' if(!$extension);
  my $filePattern="\\.$extension\$";

  opendir(DIR,'.');
  my @mainlist=readdir(DIR);
  closedir(DIR);

  my $wczytanoPlikow=0;
  my $daneRekordu='';

  DIRLOOP:
  foreach my $dir (@mainlist){
    if( $dir eq 'studenci'){
      $filePattern="-(\\d+)\\.\\w*\\.?$extension\$" if($edycja);
    }elsif($dir =~ /^studenci-(\d)$/){
      next DIRLOOP if($edycja && $edycja != $1);
      $filePattern="\\.$extension\$";
    }else{
      next DIRLOOP;
    }
    if(-d $dir){
      opendir(DIR,$dir);
      my @filelist=readdir(DIR);
      closedir(DIR);

      FILELOOP:
      foreach my $file(@filelist){
        if($file =~ $filePattern){
          next FILELOOP if($edycja && $dir eq 'studenci' && $edycja != $1);
          print "Czytam studentw z pliku $file, par->{StanListy}=$par->{StanListy}=\n";
          $self->merge("$dir/$file",$par,'student',$masterRecord);
          $wczytanoPlikow++;
#          Zdecydowanie powinna tu by ta druga wersja, ale bya wykomentowana, a zostawiona pierwsza
#          W jakiej sytuacji wida wic potrzebuj pierwszej - trzeba wyjani w jakiej
#          $daneRekordu=$par->{DaneRekordu};
          if($par->{StanListy} ne 'aneks' && $par->{StanListy} ne 'suplement'){
print "zmieniam dane rekordu na $par->{DaneRekordu} \n";
            $daneRekordu=$par->{DaneRekordu};
          }
        }
      }
    }
  }
  # Jeli nic nie udao si przeczyta, prbujemy jeszcze po staremu
  if($wczytanoPlikow == 0 && -r $oldStyleStudentsFile){
    $self->read("$dir/$oldStyleStudentsFile",$par,'student',$masterRecord);
    $wczytanoPlikow++;
  }
  # A w ostatecznoci dajemy komunikat
  if($wczytanoPlikow == 0){
    my $komunikat=<<END

Nie znaleziono katalogu studenci ani pliku ze studentami
o nazwie $oldStyleStudentsFile (stara konwencja)
Plik w formacie *.csv otrzymany z dziekanatu naley
umieci w katalogu studenci.
END
    ;
    $raport->wymalujError($komunikat);
  }
  $RecordBase::warnOnFindingNewRecords=1;
  return $daneRekordu;
}

################################
#
# Funkcja pomocnicza ustalajca numery examId wg ktrych
# studenci s identyfikowani na egzaminie.
#

sub assignExamId{
  my ($self,$par,$raport,$oknoZadania)=@_;
  my $studentsExamIdFile=$par->{nazwaBazowaTestu}.'.sei';

  # Docelowo plik *.sei powinien by w auxil, gdzie bdzie bardziej chroniony
  # dlatego dopisujemy auxil, chyba e ju jest w katalogu gwnym
  if(!(-e $studentsExamIdFile)){
    mkdir('auxil',0660) if (!(-d 'auxil'));
    $studentsExamIdFile="auxil/$studentsExamIdFile";
  }

  # Dla zgodnoci ze star wersj

  # Przy generowaniu testu dajemy szans wygenerowania examId od nowa,
  # co przydaje si tylko w sytuacji, gdy gwatownie zmniejszya si lista
  # studentw i nie chcemy mie duej iloci mieci, a gotowi jestemy
  # nowe testy wydrukowa od nowa

#  my $dopisujemy=(lc($par->{caller}) eq lc('robFormularzeOdpowiedzi'):'NIE'?'TAK');
  my $dopisujemy='TAK';
  # Mergujemy dane o examId, ktre zostay okrelone wczeniej (jeli takie s)
  if(-e $studentsExamIdFile && lc($par->{caller}) eq lc('robFormularzeOdpowiedzi')){
#  my $dopisujemy='NIE';
#  # Mergujemy dane o examId, ktre zostay okrelone wczeniej (jeli takie s)
#  if(-e $studentsExamIdFile && lc($par->{caller}) ne lc('robFormularzeTestu')){
    $dopisujemy=$oknoZadania->Dialog(
       -title => 'Powtrna generacja formularzy odpowiedzi',
       -text => "Studentom przypisano ju numery egzaminacyjne (sei).\n".
                "Czy zachowa istniejce numery?\n".
                "Jeli formularze odpowiedzi nie byy jeszcze drukowane ".
                "najlepiej wybra odpowied NIE.\n".
                "Zachowanie istniejcych numerw pozwala wykorzysta ju ".
                "wydrukowane formularze odpowiedzi i jedynie dodrukowa ".
                "formularze dla nowych studentw, ktre umieszczone zostan ".
                "na kocu pliku.\n".
                "Czy zachowa istniejce numery?",
       -default_button => 'TAK',
       -buttons => ['TAK','NIE'],
     )->Show();
  }
  if(-e $studentsExamIdFile && $dopisujemy eq 'TAK'){
    print <<END
Czytam liste juz przypisanych examId.
Skresleni studenci zostana wyszczegolnieni jako nowe rekordy.
Ich formularze mozna zniszczyc.
END
    ;
    $self->merge($studentsExamIdFile,$par,'student');
  }

  # Wyznaczamy maksymalny przypisany examId
  # i oznaczamy studentw, ktrzy jeszcze nie maj okrelonego examId
  my $maxExamId=0;
  while(my($studKey,$stud)=each(%$self)){
    if(defined($stud->{examId})){
     $maxExamId=($maxExamId > $stud->{examId} ? $maxExamId : $stud->{examId});
    }else{
      $stud->{examId}='none';
    }
  }

  # Teraz trzeba jeszcze wygenerowa numery dla nowych studentw, ale
  # nie robimy tego przy ocenianiu testw, bo ju i tak byoby na to za pno
  return $maxExamId if ( lc($par->{caller}) eq 'ocentest' );

  my $newStudentRB=$self->select('examId','none');
  my $oldMaxExamId=$maxExamId;
  # Szykujemy posortowana liste studentow bez examId
  my @toSort=();
  while(my($recordKey,$recordRef)=each(%$newStudentRB)){
    my $sortkey="$recordRef->{nazwisko} $recordRef->{imie} $recordRef->{indeks}";
    push @toSort,"$sortkey***$recordKey";
  }
  my @sorted=@{polSort \@toSort};
  my $lp=0;


  # Przypisujemy nowym studentom kolejne numery examId
  foreach my $elem (@sorted){
    my ($sortkey,$recordKey)=split(/\*\*\*/,$elem);
    if(++$maxExamId >= $par->{pierwszyZapasowyExamId}){
      $egzaminator::biezacyRaport->zakoncz("Brak moliwoci wygenerowania nowego examId. Przekroczono $par->{pierwszyZapasowyExamId}.\n");
    }
    $self->{$recordKey}->{examId}=$maxExamId;
  }

  # Zapisujemy now wersj pliku z numerami examId, ale tylko w przypadku obsugi generowania
  # formularzy odpowiedzi

  return $maxExamId if ( lc($par->{caller})  ne lc('robFormularzeOdpowiedzi') );
  my @parFieldsArray=('DaneRekordu');
  my @fieldsOrderArray=('nazwisko','imie','indeks','numerOsoby','examId');
  my @sortingFieldsArray=('nazwisko','imie','indeks');
  # Informujemy o wynikach dodawania studentow
  my $oldMaxExamIdPlusOne=$oldMaxExamId+1;
  if($dopisujemy eq 'TAK'){
    if($maxExamId>$oldMaxExamId){
      $raport->wymaluj("Nowi studenci maj numery egzaminacyjne od $oldMaxExamIdPlusOne do $maxExamId\n");
      $raport->wymaluj("Tylko te formularze wystarczy dodrukowa\n");
      $self->write($studentsExamIdFile,$par,\@parFieldsArray,\@fieldsOrderArray,\@sortingFieldsArray);
    }else{
      $raport->wymaluj("Nie znaleziono nowych studentw\n");
    }
  }else{
     $raport->wymaluj("Studenci maj numery egzaminacyjne od $oldMaxExamIdPlusOne do $maxExamId\n");
     $raport->wymaluj("Jeli byy ju wydrukowane formularze odpowiedzi, trzeba je zniszczy\n");
     $self->write($studentsExamIdFile,$par,\@parFieldsArray,\@fieldsOrderArray,\@sortingFieldsArray);
  }
  return $maxExamId;
}

################################

1;
