###############################
#
# Klasa SelektorKorektora
#
# (c) Marian Mrozek 2008
#
#  Klasa udostpnia rekordy do interaktywnej korekty
#  efektw skanowania


package SelektorKorektora;

use Tk;
use strict;
use utilities;
require Tk::ROText;
use Korektor;


################################
#
# Konstruktor
#

sub new{
  my $class = shift;
  my $self = {};
  bless($self,$class);
  (
    $self->{mainWindow},$self->{par},$self->{referencjaZadania}
  ) = @_;
  $self->init();

  return $self;
}


###########################################################################
#
# Tworzy gwne okno korektora i wypenia je pocztkow treci
#

sub configWindow{
  my $self = shift;

  # Tworzymy gwne okno
  $self->{ramka} = $self->{mainWindow}->Toplevel();
  $self->{ramka}->minsize(600,400);
  $self->{ramka}->title("Selektor korektora");
  my $buttonsFrame=$self->{ramka}->Frame()->place(-relx => 0.0, -rely => 0.8, -relwidth => 1.0, -relheight => 0.2);
  # Tworzymy potrzebne widgety
  $self->{lb}=$self->{ramka}->Scrolled("Listbox",-scrollbars=> "e",-selectmode => "single")->
    place(-relx => 0.05, -rely => 0.05, -relwidth => 0.9, -relheight => 0.75);
  for(my $i=0;$i<@{$self->{nazwyBazowe}};++$i){
    $self->{lb}->insert('end',$self->{nazwyBazowe}->[$i]);
  }

  $self->{lb}->selectionSet(0);

  $buttonsFrame->Button(-text => "Koryguj", -command => ['koryguj', $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);
  $buttonsFrame->Button(-text => "Usu", -command => ['usun' , $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);
  $buttonsFrame->Button(-text => "Zapisz i zakocz", -command => ['zakoncz' , $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);
  $buttonsFrame->Button(-text => "Anuluj", -command => ['anuluj' , $self] )->
    pack(-side => 'left', -fill => 'x', -expand => 1);

}

################################
#
# inicjalizacja
#

sub init{
  chdir("skaner");
  my $self = shift;
  $self->{rekordy}=[];            # Kolejne linie pliku skaner.txt
  $self->{nazwyBazowe}=[];        # Powizanie: linie pliku skaner.txt (numerowane od zera) -> nazwa bazowa pliku
  $self->{pozycjeWWiazce}={};     # Powizanie: nazwa bazowa pliku -> linie pliku skaner.txt (numerowane od zera)
  if(!open(SK,"<records.txt")){
    chdir('..');
    die("Failed to open records.txt");
  }else{
    print "Opened records.txt for reading\n";
  }
  my $cnt=0;
#print"self->{rekordy}=$self->{rekordy}=\n";
  while(<SK>){
#print "   Analyzing line $_";
#    chop;
    $self->{rekordy}->[$cnt]=$_;
    $self->{nazwyBazowe}->[$cnt++]=$1 if(/^([^;]+);/);
  }
  close(SK);
}


###########################################################################
#
# obsuga przycisku 'Zakocz'
#

sub zakoncz{
  my $self = shift;
  my $answer=$self->{ramka}->Dialog(
       -title => 'Zapis zmian',
       -text => 'Czy zapisa dokonane zmiany?',
       -default_button => 'Nie',
       -buttons => ['Tak','Nie'],
     )->Show();
  if($answer eq 'Tak'){
    if(!open(SK,">records.txt")){
      chdir('..');
      die("Failed to open records.txt");
    }
    for(my $i=0;$i<@{$self->{rekordy}};++$i){
      print SK $self->{rekordy}->[$i] if($self->{rekordy}->[$i]);
    }
    close(SK);
  }
  chdir("..");
  ${$self->{par}->{referencjaDoSelektoraKorektora}}=undef;
  $self->{ramka}->destroy();
}

###########################################################################
#
# obsuga przycisku 'Anuluj'
#

sub anuluj{
  my $self = shift;
  chdir("..");
  ${$self->{par}->{referencjaDoSelektoraKorektora}}=undef;
  $self->{ramka}->destroy();
}

###########################################################################
#
# obsuga przycisku 'Usu'
#

sub usun{
  my $self = shift;
  my $sel=$self->{lb}->curselection();
  my $answer=$self->{ramka}->Dialog(
       -title => 'Usuwanie rekordu',
       -text => 'Czy usun rekord '.$self->{nazwyBazowe}->[$sel]."?\n",
       -default_button => 'Nie',
       -buttons => ['Tak','Nie'],
     )->Show();
  if($answer eq 'Tak'){
    $self->{rekordy}->[$sel]=undef;
    $self->{lb}->delete($sel);
  }
}

###########################################################################
#
# obsuga przycisku 'Koryguj'
#

sub koryguj{
  my $self = shift;
  my @selarray=$self->{lb}->curselection();
  my $sel=$selarray[0];
  if(!$self->{korektor}){
#print "Calling new Korektor with $self,$sel,$self->{referencjaZadania},$self->{nazwyBazowe}->[$sel]=\n";
    $self->{korektor}=new Korektor($self,$sel,$self->{referencjaZadania},$self->{nazwyBazowe}->[$sel]);
    $self->{par}->{referencjaDoKorektora}=\$self->{korektor};
    $self->{korektor}->configWindow();
  }
}

###########################################################################

1;