###############################
#
# Klasa starterEgzaminatora
#
# (c) Marian Mrozek 1996-2003
#
# Klasa uywana do uruchamiania egzaminatora
# w wersji graficznej. Pozwala na wybranie
# katalogu, w ktrym odby si ma selekcja
#

package starterEgzaminatora;

use Tk;
use FileDialog;
use DirSelect;
use utilities;
use strict;
require Tk::ROText;
use Win32::TieRegistry( Delimiter=>"#", ArrayValues=>0 );
use Win32::Process;

use copyDirTree;            # klasa copyDirTree
use nowyEgzamin;            # klasa nowyEgzamin

my $wariant='';


################################
#
# Konstruktor
#

sub new{
  my $class = shift;
  my $self = {};
  bless($self,$class);
  $self->{Bin} = shift;

  $self->{par}={};
  $self->{par}->{Bin}=$self->{Bin};
  $self->{par}->{dir}=pwd();

  my $egzConfFile="egzaminator.conf" ;
  my $egzConfFileDir=(-r "c:\\$egzConfFile" ? "c:\\" : "$self->{Bin}/egzaminator");
  readConfiguration($egzConfFileDir,1,$self->{par},$egzConfFile);

# Przeniesione do $self->select()
# Czytam katalog biecy z pliku statusEgzaminatora
#  if(-r "$self->{Bin}/egzaminator/statusEgzaminatora"){
#    open(IN,"<$self->{Bin}/egzaminator/statusEgzaminatora");
#    $self->{par}->{'katalogBiezacy'}=$self->{'katalogBiezacy'}=<IN>;
#    close(IN);
#  }

  return $self;
}

################################
#
# Wydobywacz cieki
#

sub getPath{
  my $arg=shift;
  my $sys=$ENV{SystemRoot};
  $arg =~ s/\"//g;
  $arg =~ s/%1//;
  $arg =~ s/\/e//;
  $arg =~ s/%SystemRoot%/$sys/;
  if($arg =~ /^(.*)\s+$/){
    $arg=$1;
  }
  return $arg;
}

###########################################################################
#
# Odczytuje plik z ostatnimi przetwarzanymi egzaminami
#

sub readStatus{
  my $self = shift;
  open(IN,"<$self->{Bin}/egzaminator/statusEgzaminatora");
  $self->{licznikKatalogow}=0;
  $self->{'wykazKatalogow'}=[];
  while(<IN>){
    s/\n//;
    if(-d $_){
      $self->{'wykazKatalogow'}->[$self->{licznikKatalogow}++]=$_;
    }
  }
  close(IN);
  $self->{par}->{'katalogBiezacy'}=$self->{'katalogBiezacy'}=$self->{'wykazKatalogow'}->[0];
}

###########################################################################
#
# Zapisuje plik z ostatnimi przetwarzanymi egzaminami
#

sub writeStatus{
  my $self = shift;
  open(OUT,">$self->{Bin}/egzaminator/statusEgzaminatora");
  if(-d $self->{par}->{'katalogBiezacy'}){
    print OUT "$self->{par}->{'katalogBiezacy'}\n";
  }
  $self->{licznikKatalogow}=10 if $self->{licznikKatalogow}>10;
  for(my $i=0;$i<$self->{licznikKatalogow};++$i){
    if($self->{'wykazKatalogow'}->[$i] ne $self->{par}->{'katalogBiezacy'}){
      print OUT "$self->{'wykazKatalogow'}->[$i]\n";
    }
  }
  close(OUT);

}

###########################################################################
#
# Dodaje menu
#

sub addMenu{
  my $self = shift;

  my $mw=$self->{starterWindow};
  $mw->configure(-menu => my $menubar = $self->{menu} = $mw->Menu);
  $self->{egzMenu}=$menubar->cascade(-label => 'Egzamin');

  $self->{utworzMenu}=$self->{egzMenu}->command(-label => "Nowy egzamin", -command => ['newExam' => $self] );
  $self->{wybierzMenu}=$self->{egzMenu}->command(-label => "Wybierz egzamin", -command => ['selectViaMenu' => $self] );
  $self->{kopiujMenu}=$self->{egzMenu}->command(-label => "Powiel egzamin", -command => ['importExam' => $self] );
  $self->{konfigurujMenu}=$self->{egzMenu}->command(-label => "Ustawienia egzaminatora", -command => ['konfiguruj' => $self] );
  $self->{pomocMenu}=$self->{egzMenu}->command(-label => "Pomoc", -command => ['help' => $self] );
  $self->{zakonczMenu}=$self->{egzMenu}->command(-label => "Zakocz", -command => ['finish' => $self] );
  $self->{egzMenu}->separator();
  $self->readStatus();
  for(my $i=0;$i<$self->{licznikKatalogow};++$i){
    $self->{egzMenu}->command(-label => $self->{'wykazKatalogow'}->[$i], -command => ['selectLast' => $self, $self->{'wykazKatalogow'}->[$i] ] );
  }
}

###########################################################################
#
# Tworzy gwne okno startera egzaminatora i wypenia je pocztkow treci
#

sub wyswietlOkno{
  my ($self)=@_;
  $self->{starterWindow} = MainWindow->new;
  $self->{starterWindow}->minsize(400,50);
  $self->{starterWindow}->title("Egzaminator 2008-09-01");
  $self->sprawdzSciezki();
#  my @fam=sort $self->{starterWindow}->fontFamilies;
#  foreach my $f(@fam){
#    print "$f\n";
#  }

  $self->{font}=$self->{starterWindow}->fontCreate('heading', -size => 14, -weight => 'bold');
#  $self->{font}=$self->{starterWindow}->fontCreate('heading', -size => 14, -weight => 'bold', -family => 'system');
#  $self->{font}=$self->{starterWindow}->fontCreate('heading', -size => 14, -weight => 'bold', -family => 'Courier New CE');

  $self->addMenu();

  my $copyright=$self->{starterWindow}->Text(-width => 40, -height => 1, -relief => 'ridge', -background => 'gray')->
    pack(-side => 'bottom', -anchor => 'e',, -fill => 'x');


  $copyright->insert('0.0',"(c) Marian Mrozek, Krakw 1996-2008");
}

###########################################################################
#
# Dokonuje wyboru katalogu, w ktrym bdzie pracowa egzaminator
#

sub select{
  my ($self,$examDir)=@_;

  my $dir;
  if(!$examDir){
    # Wybieramy katalog przy uyciu DirSelect
    $self->{DSRef} = $self->{starterWindow}->DirSelect(-title => "Podaj katalog");
    $dir = $self->{DSRef}->Show($self->{par}->{'katalogPoczatkowy'});
  }else{
    $dir=$examDir;
  }
  if(!(-d $dir)) { return;}

  $self->{par}->{'katalogBiezacy'}=$self->{'katalogBiezacy'}=$dir;
  if($dir){
    chdir($dir);
  }else{
    return;
  }

  # To jest nowe podejcie, wypada na wieo przeczyta konfiguracj egzaminatora
  my $egzConfFile="egzaminator.conf" ;
  my $egzConfFileDir=(-r "c:\\$egzConfFile" ? "c:\\" : "$self->{Bin}/egzaminator");
  readConfiguration($egzConfFileDir,1,$self->{par},$egzConfFile,1);
  $self->sprawdzSciezki();
  # oraz konfiguracj egzaminu
  if(-r 'conf'){
    # Wymuszamy czytanie konfiguracji od nowa ostatnim parametrem ustawionym na 1
    readConfiguration('.',1,$self->{par},'conf',1);
    if(!$self->{par}->{kurs}){
      my $answer=$self->{starterWindow}->Dialog(
         -title => 'Zy katalog',
         -text => 'W wybranym katalogu jest plik konfiguracyjny, ale nie jest to plik konfiguracyjny egzaminatora lub nie podano nazwy przedmiotu (parametr kurs)',
         -default_button => 'OK',
         -buttons => ['OK'],
       )->Show();
      return;
    }
    $starterEgzaminatora::wariant=$self->{par}->{wariant};
    if(!$self->{par}->{wariant} && $self->{par}->{domyslnyWariant}){
      $starterEgzaminatora::wariant=$self->{par}->{wariant}='WSB' if($self->{par}->{sala} =~ /[ABCDE]\d\d\d/);
      $starterEgzaminatora::wariant=$self->{par}->{wariant}='UJ' if($self->{par}->{sala} =~ /GP?\d\./);
    }
  }else{
    my $answer=$self->{starterWindow}->Dialog(
         -title => 'Nowy egzamin',
         -text => "W wybranym katalogu $dir brak poprawnego pliku konfiguracyjnego. Czy chcesz utworzy nowy egzamin?",
         -default_button => 'Tak',
         -buttons => ['Tak','Nie'],
       )->Show();
    if($answer eq 'Tak'){
      $self->{par}->{przebiegInicjalizacyjny}=1;
      mkdir('studenci',0660) if (!(-d 'studenci'));
      mkdir('tematy',0660) if (!(-d 'tematy'));
    }else{
      return;
    }
  }
  $self->{par}->{przebiegOkienkowy}=1;
  $self->{par}->{numerDomyslnyEdycji}=$main::opt_e;
  my $egz=new egzaminator($self);

  $self->{egzMenu}->configure(-state => 'disabled');
  $egz->wyswietlOkno();
}

###########################################################################
#
# Pozwala wybra katalog biecy poprzez menu
#

sub selectViaMenu{
  my ($self)=@_;
  $self->select("");
}

###########################################################################
#
# Dokonuje wyboru ostatniego katalogu, w ktrym pracowa egzaminator
# jako katalogu biecego
#

sub selectLast{
  my ($self,$lastExam)=@_;
  $self->select($lastExam);
}

###########################################################################
#
# Tworzy kopi istniejcego katalogu z egzaminem
# w podanym miejscu
# (tylko plik conf, i zawarto katalogu tematy)
#

sub importExam{
  my ($self)=@_;

  $self->{DSRef}=$self->{starterWindow}->DirSelect(-title => "Wybierz katalog rdowy");
#  my $dir = $self->{DSRef}->Show($self->{par}->{'katalogBiezacy'});
  my $dir = $self->{DSRef}->Show($self->{par}->{'katalogPoczatkowy'});
  if(!$dir){ return;}


  my $cETDSel=new nowyEgzamin($self,$dir);
  $cETDSel->process();
}

###########################################################################
#
# Tworzy nowy katalog egzaminu
#

sub newExam{
  my ($self)=@_;

  my $cETDSel=new nowyEgzamin($self,$self->{par}->{Bin}."/egzaminator/pustyEgzamin",1);
  $cETDSel->process();
}

###########################################################################
#
# Uruchamia konfigurator egzaminatora
#

sub konfiguruj{
  my ($self)=@_;

  my $confFile=(-r "c:/egzaminator.conf" ? "c:/egzaminator.conf" : "$self->{Bin}/egzaminator/egzaminator.conf");
#print "Got $confFile\n";
  # my $confFile="$self->{Bin}/egzaminator/egzaminator.conf";
  $self->{konfigurator} = new konfigurator($self->{starterWindow},$confFile,undef,$self->{par},$self,100,100);
  $self->{egzMenu}->configure(-state => 'disabled');
  $self->{konfigurator}->configWindow();
}

###########################################################################
#
# Sprawdza, czy istniej programy zdefiniowane w pliku konfiguracyjnym
# Sygnalizuje bd, jeli s ze, ustawia domylne jeli wartoci s puste.
#

sub sprawdzSciezki{
  my ($self)=@_;

  $Registry->Delimiter("/");
  my $swKey= $Registry->{"LMachine/Software/"};

  my $result=1;
  if(!$self->{par}->{CzytnikPDF} || !(-r $self->{par}->{CzytnikPDF})){
    my $access=$swKey->{'Adobe/Acrobat Reader/8.0/InstallPath/'}->{''}.'\\AcroRd32.exe';
#print "CzytnikPDF=$access=\n";
    if($access && -r $access){
      $self->{par}->{'CzytnikPDF'}=$access;
    }else{
      $self->{starterWindow}->Dialog(
         -title => '',
         -text => 'Nie udao si zlokalizowa czytnika pdf. Podaj poprawn lokalizacj programu w ustawieniach egzaminatora.',
         -default_button => 'OK',
         -buttons => ['OK'],
       )->Show();
      $result=0;
    }
  }
  if(!$self->{par}->{CzytnikHTML} || !(-r $self->{par}->{CzytnikHTML})){
    my $access=$swKey->{'Classes/Applications/iexplore.exe/shell/open/command/'}->{''};
    $access=getPath($access);
#print "CzytnikHTML=$access=\n";
    if($access && -r $access){
      $self->{par}->{'CzytnikHTML'}=$access;
    }else{
      $self->{starterWindow}->Dialog(
         -title => '',
         -text => 'Nie udao si zlokalizowa czytnika html. Podaj poprawn lokalizacj programu w ustawieniach egzaminatora.',
         -default_button => 'OK',
         -buttons => ['OK'],
       )->Show();
      $result=0;
    }
  }
  if(!$self->{par}->{EdytorCSV} || !(-r $self->{par}->{EdytorCSV})){
    my $access=$swKey->{'Classes/Applications/EXCEL.EXE/shell/open/command/'}->{''};
    $access=getPath($access);
#print "EdytorCsv=$access\n";
    my $access2=$swKey->{'Classes/Applications/notepad.exe/shell/open/command/'}->{''};
    $access2=getPath($access2);
#print "EdytorCsv2=$access2\n";
    if($access && -r $access){
      $self->{par}->{'EdytorCSV'}=$access;
    }elsif($access2 && -r $access2){
      $self->{par}->{'EdytorCSV'}=$access2;
    }else{
      $self->{starterWindow}->Dialog(
         -title => '',
         -text => 'Nie udao si zlokalizowa edytora csv. Podaj poprawn lokalizacj programu w ustawieniach egzaminatora.',
         -default_button => 'OK',
         -buttons => ['OK'],
       )->Show();
      $result=0;
    }
  }
  if(!$self->{par}->{Edytor} || !(-r $self->{par}->{Edytor})){
    my $access=$swKey->{'Classes/Applications/notepad.exe/shell/open/command/'}->{''};
    $access=getPath($access);
#print "Edytor=$access\n";
    if($access && -r $access){
      $self->{par}->{'Edytor'}=$access;
    }else{
      $self->{starterWindow}->Dialog(
         -title => '',
         -text => 'Nie udao si zlokalizowa czytnika tekstu. Podaj poprawn lokalizacj programu w ustawieniach egzaminatora.',
         -default_button => 'OK',
         -buttons => ['OK'],
       )->Show();
      $result=0;
    }
  }
  if(!$self->{par}->{Cmd} || !(-r $self->{par}->{Cmd})){
    $self->{starterWindow}->Dialog(
       -title => '',
       -text => 'Nie udao si zlokalizowa egzekutora komend. Podaj poprawn lokalizacj programu w ustawieniach egzaminatora.',
       -default_button => 'OK',
       -buttons => ['OK'],
     )->Show();
    $result=0;
  }
  if(!$self->{par}->{TexBin} || !(-r $self->{par}->{TexBin})){
    my $path=$swKey->{'MiK/MiKTeX/CurrentVersion/MiKTeX/Install Root'}.'\\miktex\\bin\\';
#print "texbin=$path=\n";
    if($path && -r $path){
      $self->{par}->{'TexBin'}=$path;
    }else{
      $self->{starterWindow}->Dialog(
         -title => '',
         -text => 'Nie udao si zlokalizowa cieki dostpu do binariw MiKTeXa. Podaj poprawn ciek w ustawieniach egzaminatora.',
         -default_button => 'OK',
         -buttons => ['OK'],
       )->Show();
      $result=0;
    }
  }
  my $locKatalogPoczatkowy="$ENV{'APPDATA'}\\MojeEgzaminy";
  if(!$self->{par}->{katalogPoczatkowy}){
    if(!(-d $locKatalogPoczatkowy)){
      my $ans=$self->{starterWindow}->Dialog(
         -title => '',
         -text => 'Brak katalogu pocztkowego. Czy utworzy katalog \'MojeEgzaminy\'?',
         -default_button => 'TAK',
         -buttons => ['TAK','NIE'],
      )->Show();
      if($ans eq 'TAK'){
        if(!(-d $ENV{'APPDATA'})){
          my $command="mkdir \"$ENV{'APPDATA'}\"";
#          print "Doing 1: $command\n";
          `$command`;
        }
        my $command="mkdir \"$locKatalogPoczatkowy\"";
#        print "Doing 2: $command\n";
        `$command`;
      }
    }
    $self->{par}->{katalogPoczatkowy}=$locKatalogPoczatkowy;
  }
  if(!(-d $self->{par}->{katalogPoczatkowy})){
    my $ans=$self->{starterWindow}->Dialog(
       -title => '',
       -text => 'Podany katalog pocztkowy nie istnieje. Podaj poprawn ciek w ustawieniach egzaminatora.',
       -default_button => 'OK',
       -buttons => ['OK'],
     )->Show();
  }
  if(!$result){
#    $self->konfiguruj();
  }
}


###########################################################################
#
# Wywietla okno z pomoc
#

sub help{
  my ($self,$par)=@_;
  my $command="\"$self->{par}->{CzytnikHTML}\" $self->{par}->{Bin}\\egzaminator\\doc\\doc.html";
#  my $out=`$command`;

  # Create the process object for a detached DOS window
  # and execute command there
  Win32::Process::Create($Win32::Process::Create::ProcessObj,
    $self->{'par'}->{'Cmd'}, #
    "/c $command", #
    0, # Dont inherit.
    CREATE_NEW_CONSOLE,
    "."
  ) or
  die Win32::FormatMessage( Win32::GetLastError() );
}

###########################################################################
#
# Koczy prac
#

sub finish{
  my ($self,$par)=@_;
  $self->{starterWindow}->destroy();
# Przeniesione do egzaminator->done()
#  open(OUT,">$self->{Bin}/egzaminator/statusEgzaminatora");
#  print OUT $self->{'katalogBiezacy'};
#  close(OUT);
}


###########################################################################

1;