################################
#
# klasa  examTestGenerator
# pochodna od examTest
#
# klasa czyta zestaw pyta do testu
# (ewentualnie podzielony na grupy)
# i generuje losowe testy

package examTestGenerator;

@ISA = ("examTest");

use examTest;
use strict;
use utilities;
use binary;
use student;

use XML::TreeBuilder;
use XML::Element;
use polChar;
use textUtil;
# use redoLineNumberLabels;



################################
#
#  Musimy kodowa polskie litery i \n ze wzgldu na XML::TreeBuilder
sub xcode{
  my ($self,$arg)=@_;
  $arg =~ s/\012//g if($self->{conf}->{sysop} eq 'WIN32');
  $arg=pol2esc($arg);
  return $arg;
}
sub xdecode{
  my ($self,$arg)=@_;
  $arg =~ s/\012/\012\015/g if($self->{conf}->{sysop} eq 'WIN32');
  $arg=esc2pol($arg);
  return $arg;
}

################################
#
#
sub redoLineNumberLabels{
  my ($self,$text,$dir)=@_;
  my $newText='';
  my $listing='';
  my $sourceName;
  my $commentSign='//';

  if($text =~ /^(.*)\n\s*\\begin\{listing\}\{([.\w]+)\}\s*\n(.*)\\end\{listing\}\s*\n(.*)$/s){
    my @lines;
    my $noLines;
    my $preface=$1;
    $sourceName=$2;
    my $source=$3;
    my $quiz=$4;

    if($source =~ /\S/){
      @lines=split(/\n/,$source);
    }else{
      open(IN,"$dir/$sourceName") || die("Brak kodu rdowego $sourceName w pliku *.xml i w pliku zewntrznym.");
      @lines = <IN>;
      close(IN);
      $source = join('\n',@lines);
    }
    $noLines=$#lines;
    my %lineRefs;
    if($lines[$noLines] =~ /\S/){++$noLines;} # Jeli ostatnia linia jest niepusta, to j doliczamy
    my $maxLength;
    for(my $i=0;$i<$noLines;++$i){
      if($lines[$i] =~ /^(.*)(\/\/|\#|;)\?(\w+)\?/){
        my $line=$1;
        $commentSign=$2;
        my $key=$3;
        if(defined($lineRefs{$key})){
          $self->{raport}->wymalujError("Powtrzona definicja etykiety =$key=");
        }else{
          my $n=$i+1;
          $n=" $n" if($n<10);
          $lineRefs{$key}=$i+1;
          $lines[$i]=$line;
        }
      }
      $lines[$i]=eject_right_blanks($lines[$i]);
      $maxLength=length($lines[$i]) if($maxLength<length($lines[$i]));
    }
    ++$maxLength;
    for(my $i=0;$i<$noLines;++$i){
      my $n=$i+1;
      $n=" $n" if($n<10);
      if($lines[$i] =~ /^(.*\S)\s+$/){
        $lines[$i]=$1;
      }
      $listing.="$lines[$i]";
      $listing.=' 'x ($maxLength-length($lines[$i]));
      $listing.="$commentSign($n)\n";
      $lines[$i]="$n $lines[$i]";
    }
    while($quiz =~ /^(.*)\?(\w+)\?(.*)$/s){
      my ($left,$key,$right)=($1,$2,$3);
      if(defined($lineRefs{$key})){
        $self->{raport}->wymaluj("Zmieniono =$key=\n");
        $quiz="$left($lineRefs{$key})$right";
      }else{
        $self->{raport}->wymalujError("Nie odnaleziono etykiety =$key=\n");
       last;
      }
    }
    $source=join("\n",@lines);
    $newText=<<END
$preface
\\begin{verbatim}
$source
\\end{verbatim}
$quiz
END
    ;
    $text=$newText;
  }else{
    if($text =~ /^(.*)\n\s*\\begin\{listing\}/){
      $self->{raport}->wymalujError("Niepoprawny nagwek listingu");
    }
  }
  return ($text,$listing,$sourceName);
}

################################
#
# Funkcja czyta plik rdowy z testem w formacie XML
# i generuje odpowiedni obiekt
#
sub readXML{
  my ($self,$dir,$fileName)=@_;
  my $file="$dir/$fileName";
  my $g=$self->{groupCounter}; # licznik grup
  my $tot_p=$self->{problemCounter}; # zbiorczy licznik odpowiedzi
  my $p;     # licznik problemw
  my $o;     # licznik odpowiedzi
  my ($keyname,$value);

  my $tree = XML::TreeBuilder->new();
  open(IN,"<$file") || $self->{raport}->zakoncz("Failed to open file $file\n");
  my $size = -s $file;
  my $text;
  read(IN,$text,$size);
  close(IN);

  # Numerujemy linie ewentualnego programu oraz zamieniamy etykiety postaci ?nazwa?
  # na numery linii
  my ($listing,$sourceName);
  ($text,$listing,$sourceName)=$self->redoLineNumberLabels($text,$dir);
  if($sourceName){
    open(OUT,">$dir/$fileName".'n') || $self->{raport}->zakoncz("Failed to open file $fileName.xmln\n");
    print OUT $text;
    close(OUT);

    open(OUT,">$dir/st_$sourceName") || $self->{raport}->zakoncz("Failed to open file st_$sourceName\n");
    print OUT $listing;
    close(OUT);
  }

  # Kodujemy polskie litery i znaki zmiany linii ze wzgldu na XML::TreeBuilder
  $text=$self->xcode($text);
  # Budujemy drzewo dokumentu
  eval{ $tree->parse($text);};
  if($@){
    my $line;
    my $column;
    if($@ =~ /at line (\d+), column (\d+)/){
      ($line,$column)=($1,$2);
    }
    my $komunikat=<<END
===> Bd skadni XML w pliku $file, linia $line, kolumna $column
===> Popraw plik $file i uruchom ponownie
     Szczegowy komunikat parsera:
     $@
END
    ;
    $self->{raport}->wymalujError($komunikat);
    die("Funkcja zgosia wyjtek $@\n");
  }

  # I usuwamy pola pochodzce z pustych linii
  $tree->delete_ignorable_whitespace();

  # Wybieramy wzy
  my @groupNodes=$tree->content_list();
  my $i=0;
  # Jeli pierwszy wze nie jest referencj, to jest tekstem
  # Wtedy wyrzucamy z @groupNodes, a tekst zapamitujemy w $self->{text}
  $self->{text}=(!ref($groupNodes[0])?eject_blanks($self->xdecode(shift(@groupNodes))):'');
  # Pozostae wzy powinny by ze znacznikem <group>
  foreach my $groupNode (@groupNodes){
    # Sprawdzamy czy okrelono tag i czy jest to tag="group"
    my $tag=(ref($groupNode)?$groupNode->tag():'xxx');
    $self->{raport}->zakoncz("<group> expected\n") if($tag ne 'group');
    $g++;
    $p=0;
    # Wyznaczamy klucz do grupy
    my $gid="g".fill_with_zeros($g,4);
    # I wyznaczamy wzy w ramach grupy
    my @problemNodes=$groupNode->content_list();

    # Do testu moemy pobra tylko cz pyta z grupy (niekoniecznie wszystkie)
    # Ilo problemw do pobrania z tej grupy ustalamy na podstawie atrybutu select
    if($groupNode->attr('select') =~ /\S/){
      $self->{groups}->{$gid}->{numberOfProblemsToSelect}=$groupNode->attr('select');
    }else{
      $self->{groups}->{$gid}->{numberOfProblemsToSelect}='all';
    }
    $self->{groups}->{$gid}->{numberOfProblemsToSelect}='none' if($self->{groups}->{$gid}->{numberOfProblemsToSelect} =~ /^\s*0\s*$/);

    # Z pobranej grupy do indywidualnych testw moe by losowany tylko pewien podzbir
    # pobranej grupy. Wielko tego podzbioru ustala atrybut draw
    if($groupNode->attr('draw') =~ /\S/){
      $self->{groups}->{$gid}->{numberOfProblemsToDraw}=$groupNode->attr('draw');
    }else{
      $self->{groups}->{$gid}->{numberOfProblemsToDraw}='all';
    }
    $self->{groups}->{$gid}->{numberOfProblemsToDraw}='none' if($self->{groups}->{$gid}->{numberOfProblemsToDraw} =~ /^\s*0\s*$/);

    # Grup, w ktrej do wylosowania jest zero problemw cakowicie pomijamy
    next if ($self->{groups}->{$gid}->{numberOfProblemsToDraw} eq 'none');
    next if ($self->{groups}->{$gid}->{numberOfProblemsToSelect} eq 'none');

    # Sprawdzamy czy przed wzami z problemami jest extra informacja
    $self->{groups}->{$gid}->{text}="";
    while(!ref($problemNodes[0]) || $problemNodes[0]->tag() eq 'source'){
      # Jeli tak, to najpierw wycigamy wze z t informacj
      my $headerNode=shift(@problemNodes);
      # I wyznaczamy zawarty tam tekst stosownie do sposobu zapisu
      # Moe on by zapisany wprost lub w wle source
      my $headerText="";
      if(!ref($headerNode)){
        $headerText= $headerNode;
      }else{
        my @headerContents=$headerNode->content_list();
        $headerText= $headerContents[0];
      }
      # Dekodujemy (polskie litery i \n) i odrzucamy zewntrzne spacje
      $headerText=$self->xdecode($headerText);
      # Wyrzucamy ewentualne pocztkowe i kocowe nowe linie
      while ($headerText =~ s/^ *\n(.*)$/$1/s){};
      while ($headerText =~ s/^(.*)\n *$/$1/s){};
      # Jeli dodatkowy wze jest typu source
      if(ref($headerNode)){
          # Tu przyjdzie obrbka tekstw rdowych
      }
      $self->{groups}->{$gid}->{text}.=$headerText;
    }

    # Przygotowujemy miejsce pod problemy w tej grupie
    $self->{groups}->{$gid}->{problems}={};

    my $numberOfProblemNodes=$#problemNodes+1;
    #Ustalamy ilo problemw do pobrania
    my $numberOfProblemsToSelect=$self->{groups}->{$gid}->{numberOfProblemsToSelect};
    if(!($numberOfProblemsToSelect =~ /^\s*\d+\s*$/) || $numberOfProblemsToSelect > $numberOfProblemNodes){
      $numberOfProblemsToSelect=$numberOfProblemNodes;
    }
#    To jest w zym miejscu i le liczone, bo poszczeglne problemy mog mie
#    atrybut select=no. Przesunem w d
#    $self->{groups}->{$gid}->{numberOfSelectedProblems}=$numberOfProblemsToSelect;

    # Wybieramy losowy rosncy podcig  zbioru wzw o dugoci rwnej iloci problemw do pobrania
    my @subset=();
    for(my $i=0; $i<$numberOfProblemNodes; $i++){
      $subset[$i]=$i;
    }
    @subset=randomPerm(@subset);
    @subset=splice(@subset,0,$numberOfProblemsToSelect);
    @subset=sort{$a <=> $b}(@subset);

    # I wchodzimy w ptl przegldajc problemy
    for(my $i=0; $i<$numberOfProblemsToSelect;$i++){
      my $problemNode=$problemNodes[$subset[$i]];
      my $tag=(ref($problemNode)?$problemNode->tag():'xxx');
      $self->{raport}->zakoncz("<problem> expected\n") if($tag ne 'problem');
      next if($problemNode->attr('select') =~ /^non?e?$/);
      $p++;
      $tot_p++;
      $o=0;
      my $pid="p".fill_with_zeros($p,4);
      my @answerNodes=$problemNode->content_list();
      # Zapamitujemy tekst nagwkowy problemu
      $self->{groups}->{$gid}->{problems}->{$pid}->{text}=(!ref($answerNodes[0])?eject_blanks($self->xdecode(shift(@answerNodes))):'');
      # Zapamitujemy tekst kocowy problemu
      my $lastElement=$#answerNodes;
      $self->{groups}->{$gid}->{problems}->{$pid}->{closingText}=(!ref($answerNodes[$lastElement])?eject_blanks($self->xdecode(pop(@answerNodes))):'');
      # I przygotowujemy miejsce pod odpowiedzi w tej grupie
      $self->{groups}->{$gid}->{problems}->{$pid}->{answers}={};
      foreach my $answerNode (@answerNodes){
        next if($answerNode->attr('select') =~ /^non?e?$/);
        my $tag=(ref($answerNode)?$answerNode->tag():'xxx');
        $self->{raport}->zakoncz("<answer> expected but got <$tag>\n") if($tag ne 'answer');
        my @answerContentsNodes=$answerNode->content_list();
        $o++;
        if($#answerContentsNodes>0 || ref($answerContentsNodes[0])){
          my $n=$i+1;
          $self->{raport}->wymalujError("Error in problem $n and answer $o near: $answerContentsNodes[0] \n");
          $self->{raport}->zakoncz("<answer> tag may only contain plain text\n");
        }
        my $oid="o".fill_with_zeros($o,4);
        $self->{groups}->{$gid}->{problems}->{$pid}->{answers}->{$oid}->{text}=eject_blanks($self->xdecode($answerContentsNodes[0])); # Zapamitujemy tekst odpowiedzi
        $self->{groups}->{$gid}->{problems}->{$pid}->{answers}->{$oid}->{value}=($answerNode->attr('value') eq 'true' ? 1:0); # i warto odpowiedzi
      }
    }

    # Zmienna $p zliczya ile rzeczywicie wybralimy problemw
    $self->{groups}->{$gid}->{numberOfSelectedProblems}=$p;
    if($self->{groups}->{$gid}->{numberOfProblemsToDraw} eq 'all'){
      $self->{groups}->{$gid}->{numberOfProblemsToDraw}=$p;
    }
    print "   Do grupy $g wybrano $p problemow z $numberOfProblemNodes.\n";
    print "   Z wybranych $p problemw losowanych bdzie $self->{groups}->{$gid}->{numberOfProblemsToDraw}\n";
  }
  $self->{groupCounter}=$g; # licznik grup
  $self->{problemCounter}=$tot_p; # zbiorczy licznik problemw
}

################################
#
# Funkcja czyta plik rdowy z testem
# i generuje odpowiedni obiekt
#
sub readSRC{
  my ($self)=@_;
  my $g=0;   # licznik grup
  my $p;     # licznik problemw
  my $tot_p; # zbiorczy licznik problemw
  my $o;     # licznik odpowiedzi
  my ($keyname,$value);
  my ($gid);

  $/="***";   # Set as newline separator
print "Szukam zrodel w $self->{input}\n";
  open(INP,"<$self->{input}") || $self->{raport}->zakoncz("Nie udalo sie otworzyc pliku $self->{input} do czytania \n");

  $/="***";   # Set as newline separator
  while (<INP>){               #   remember *** is a newline separator
    if(/(.*)\*\*\*$/ms){
      $_=$1;
    }


    # Jeli rozpoznamy sowo, a potem odstp i cokolwiek to sowo traktujemy jako kluczowe
    # Jednak, jeli takie sowo ju jest w conf, ignorujemy t informacj
    if(/^(\w+)\s+(.*)$/ms){
      $keyname=$1;
      $self->{$1}=$value=eject_blanks($2) if(!($self->{par}->{$1} =~ /\S/));
    }elsif(/\S/){
      $self->{raport}->wymalujError( "examTest->read: Nierozpoznana linia: $_\n");
    }


    # Jeli napotykamy now grup problemw
    if($keyname eq 'grp'){
      $g++;
      $p=0;
      $gid="g$g";
      $self->{groups}->{$gid}->{text}=$value;       # Zapamitujemy tekst nagwkowy grupy
      $self->{groups}->{$gid}->{problems}={};       # I przygotowujemy miejsce pod pytania w tej grupie

    }

    #Jeli napotkalimy koniec pliku, podsumowujemy ilo znalezionych grup
    if($keyname eq 'end'){
       last;
#      break;
    }

    # Jeli napotkamy nowy problem
    if($keyname eq 'pro'){
      $p++;
      $tot_p++;
      $o=0;
      $self->{groups}->{"g$g"}->{problems}->{"p$p"}->{text}=$value;          # Zapamitujemy tekst nagwkowy pytania
      $self->{groups}->{"g$g"}->{problems}->{"p$p"}->{answers}={};       # I przygotowujemy miejsce pod odpowiedzi dla tego pytania
    }
    # Jesli to jest odpowiedz (poprawna lub nie)
    if($keyname eq 'odp' || $keyname eq 'ok'){
      $o++;
      $self->{groups}->{"g$g"}->{problems}->{"p$p"}->{answers}->{"o$o"}->{text}=$value; # Zapamitujemy tekst odpowiedzi
      $self->{groups}->{"g$g"}->{problems}->{"p$p"}->{answers}->{"o$o"}->{value}=($keyname eq 'ok' ? 1:0); # i warto odpowiedzi
    }
  }
  close(INP);


  #Jeli napotkalimy koniec pliku, podsumowujemy ilo znalezionych grup

  $self->{raport}->wymaluj("Znaleziono $g grup i lacznie $tot_p problemow\n\n\n");

  #Sprawdzam czy ilosc znalezionych problemw nie jest mniejsza od ilosci podanej w konfiguracji

  if($tot_p < $self->{iloscProblemow}) {
    $self->{raport}->zakoncz("W pliku z pytaniami jest za malo ($tot_p) pytan\nPowinno ich byc przynajmniej $self->{iloscProblemow} <-- ilosc podana w pliku Conf\n",
            "\nPRZERYWAM DZIALANIE PROGRAMU \n");
  }

  $/="\n";   # restore newline separator
}

################################
#
# Funkcja sprawdza rozszerzenie pliku rdowego z testem,
# i stosownie do typu czyta
#
sub read{
  my ($self)=@_;
  if($self->{par}->{formatTestu} eq 'src'){
    # Jeli format testu to src czytamy tylko jeden cile okrelony plik
    $self->readSRC();
  }else{
    # W przeciwnym razie czytamy wszystkie pliki w katalogu tematy,
    # chyba, ze jest podana edycji i jest katalog tematy-nrEdycji,
    # to wtedy szukamy w tym katalogu
    my $tematyDir=
      ( $self->{par}->{edycja} =~ /\d/ && -d "tematy-$self->{par}->{edycja}"
        ? "tematy-$self->{par}->{edycja}"
        : 'tematy'
      );
    opendir (DIRHANDLE, $tematyDir);
    my @filelist = readdir (DIRHANDLE);
    closedir(DIRHANDLE);

    $self->{groupCounter}=0;
    $self->{problemCounter}=0;
    my $fileCounter=0;
    foreach my $file (sort @filelist){
      if($file =~ /([-.\w]+).xml$/){
        print "Czytam plik $file z tematami \n";
        $fileCounter++;
        $self->readXML("$tematyDir","$file");
      }
    }
    $self->{raport}->zakoncz("Przerywam prace\nNie znaleziono zadnych plikow z testami!!!\nOczekuj plikw w formacie *.xml w katalogu tematy\n") if(!$fileCounter);
    if($self->{problemCounter} < $self->{iloscProblemow}) {
      $self->{raport}->zakoncz("W pliku z pytaniami jest za malo ($self->{problemCounter}) pytan\nPowinno ich byc przynajmniej $self->{iloscProblemow} (ilosc podana w pliku konfiguracyjnym)\n".
              "\nPRZERYWAM DZIALANIE PROGRAMU \n");
    }
    if(
      $self->{problemCounter} > $self->{iloscProblemow} &&
      $self->{par}->{wynikProcentowy} &&
      $self->{par}->{iloscPunktowNa100Procent}
    ){
      $self->{raport}->zakoncz("W pliku z pytaniami jest za duzo ($self->{problemCounter}) pytan\nPrzy podanej iloci punktw na 100% powinno ich byc dokadnie $self->{iloscProblemow} (ilosc podana w pliku konfiguracyjnym)\n".
              "\nPRZERYWAM DZIALANIE PROGRAMU \n");
    }
    print "Razem jest $self->{problemCounter} problemow\n";
  }
}

################################
#
# Funkcja zmienia opisy grup w formularzu testu  na krtkie oraz
# generuje oddzielny plik z penymi opisami (uruchamiane opcjonalnie)
#
sub redoGroupsDescriptions{
  my ($self)=@_;
  open(OUT,">$self->{groupDescriptions_output}");

  my $szerokoscWydruku=$self->{par}->{szerokoscWydruku};
  my $wysokoscWydruku=$self->{par}->{wysokoscWydruku};
  my $gornyMargines=$self->{par}->{gornyMargines};
  $szerokoscWydruku=195 if(!$szerokoscWydruku);
  $wysokoscWydruku=275 if(!$wysokoscWydruku);
  $gornyMargines=25 if(!$gornyMargines);
  $szerokoscWydruku.="mm";
  $wysokoscWydruku.="mm";
  $gornyMargines="-$gornyMargines"."mm";

  my $par=$self->{par};
  my $rozdzielczosc = ($par->{rozdzielczosc} == 10) || ($par->{rozdzielczosc} == 11)  ? "$par->{rozdzielczosc}pt" : "12pt";
  $self->{iloscKolumn} = $par->{iloscKolumn} == 2 ? 2 : 1;
  my $twoColumn = $self->{iloscKolumn} == 2 ? ",twocolumn" : "";
  my $twoSide = $par->{typDrukarki} == 2 ? ",twoside" : "";

#\\documentclass[10pt]{article}
  print OUT <<END;
\\documentclass[$rozdzielczosc$twoColumn$twoSide,a4paper]{article}
\\pagestyle{empty}
\\usepackage{polski}
\\usepackage[cp1250]{inputenc}

\\setlength{\\textheight}{$wysokoscWydruku}
\\setlength{\\topmargin}{$gornyMargines}
\\setlength{\\headsep}{2mm}
\\setlength{\\textwidth}{$szerokoscWydruku}
\\setlength{\\oddsidemargin}{-19mm}
\\setlength{\\evensidemargin}{-19mm}
\\def\\filler{\\hspace*{12mm}}
\\def\\fillerb{\\hspace*{8mm}}
\\begin{document}
{\\bf   $self->{kurs}/$self->{seria}}\\\\[2mm]
END
  ;
  my @groups=sort keys(%{$self->{groups}});
  foreach my $group (@groups){
    next if($self->{groups}->{$group}->{numberOfProblemsToDraw} eq 'none');
    $group =~ /^g(\d+)/;
    my $grpCh=($#groups<=25 ? chr(ord('A')+$1-1):$1);
    my $text=$self->{groups}->{$group}->{text};
    if(!($text =~ s/Temat\s+([A-Z])\./Temat $grpCh./)){
      $text="\\noindent{\\bf Temat $grpCh. }\n ".$text
    }
    print OUT $text;
    $self->{groups}->{$group}->{text}="{\\b{\\bf\\small === Ponisze pytania dotycz tematu $grpCh ===}}";
    print OUT "\n\n";
  }
  print OUT "\\end{document}";
  close(OUT);
}

################################
#
# Funkcja generuje tekst dla jednego egzemplarza testu
# i klucz odpowiedzi
#
sub generateCopy{
  my ($self,$numerTestu,$rozwiazany,$wszystkieProblemy)=@_;
  my ($numerEdycjiTerminu,$numerGrupySal)=($self->{par}->{numerEdycjiTerminu},$self->{par}->{numerGrupySal});
  my $form;

  my $kod=toBinForm($numerTestu,$self->{ilosc_okienek_kodu});


  my $filler=($numerTestu ? "..................................":"XXXXXXXXXXXXXX");
  my $shortFiller=($numerTestu ? ".................":"XXXXX");
  my $formularz=($rozwiazany ? "POUFNE!!!":"KONTROLNY");
  $formularz=($numerTestu ? "Formularz testu":$formularz);

  my $numerTestuSformatowany=fill_with_zeros($numerTestu,3);
  my ($ukladKolumn,$lacznikPol,$rozdzielaczPol,$lamaczLinii);
  if($self->{iloscKolumn} == 2 && $self->{typStrony} == 1){
     $lacznikPol="\\\\\n";$ukladKolumn="l"; $rozdzielaczPol="";
  }else{
     $lacznikPol="&";$ukladKolumn="ll"; $rozdzielaczPol="\\\\~~~~\\\\";
  }
#  $lamaczLinii=($self->{iloscKolumn} == 2?"\\\\~***** ":"");

  $form= <<END
\\setlength\\tabcolsep{2pt}
\\par\\noindent
\\begin{tabular*}{200mm}{$ukladKolumn}
  \\begin{tabular}{l}
    ~~~~\\\\
    {\\bf\\small Nazwisko:$filler}
    $rozdzielaczPol
    {\\bf\\small Indeks:$shortFiller}
  \\end{tabular}$lacznikPol
  \\begin{tabular}{|cl|}
  \\hline
    \\begin{tabular}{c}
      {\\bf Nr testu:} \\framebox[14mm][l]{$numerEdycjiTerminu$numerGrupySal-$numerTestuSformatowany}   \\\\
      {\\bf Kod testu}\\\\
      $kod
    \\end{tabular}
  &
  {\\footnotesize
    \\begin{tabular}{l}
      {\\bf\\Large $formularz}\\\\
      {\\bf Kurs: $self->{kurs}}\\\\
      {\\bf Test: $self->{test}}\\\\
      {\\bf Termin: $self->{data}}\\\\
      {\\bf Sala: $self->{sala}}\\\\
      {\\bf Seria: $self->{seria}}\\\\
    \\end{tabular}
  }\\\\
  \\hline
  \\end{tabular}
\\end{tabular*}
{\\bf ** \\underline{Nim rozpoczniesz, przepisz numer testu}
      \\underline{i} \\underline{kod testu na formularz odpowiedzi} **}

$self->{nagl}
\\pytcount=0
END
    ;

  # Mieszamy indeksy grup (z wyjtkiem testu wzorcowego numer 0)
  my @grpIndexes=sort keys(%{$self->{groups}});
  @grpIndexes=randomPerm(@grpIndexes) if($numerTestu != 0);
  my $grpNmb=$#grpIndexes+1;
  $self->{numberOfSelectedGroups}=$grpNmb;
  my $problCnt=0;
  my $copyAnswerKey=[];
  my $reverseNumberingKey=[];  # Uywane do statystyk ile dobrych/zych odpowiedzi na dane pytanie

  my $grpCnt=0;
  while($grpCnt<$grpNmb){
    last if($problCnt >= $self->{iloscProblemow} && !$wszystkieProblemy);
    # Ustawiamy wskanik do grupy
    my $selectedGroup=$grpIndexes[$grpCnt];
    my $a_group=$self->{groups}->{$selectedGroup};
    # Dopisujemy tekst nagwkowy tej grupy
    $form.="\n\n\\grp $a_group->{text}\n";
    # Przechodzimy do problemw w tej grupie
    my $problemsRef=$a_group->{problems};
    my @grpProblIndexes=sort keys(%$problemsRef);
    @grpProblIndexes=randomPerm(@grpProblIndexes) if($numerTestu != 0);
    my $grpProblNmb=$#grpProblIndexes+1;
    # Bdziemy zlicza ilo problemw wygenerowanych w ramach grupy
    if($a_group->{numberOfProblemsToDraw} !~ /\d/){
      $a_group->{numberOfProblemsToDraw}=$grpProblNmb;
    }
    # Po to by nie przekroczy limitu danego w pliku konfiguracyjnym
    # i ewentualnie zadanego dla kadego problemu oddzielnie
    my $grpProblLimit=(
         $a_group->{numberOfProblemsToDraw} < $self->{iloscProblemowWGrupie} ?
         $a_group->{numberOfProblemsToDraw} :
         $self->{iloscProblemowWGrupie}
    );

    my $grpProblCnt=0;
    while( $grpProblCnt<$grpProblNmb ){
    last if( ($problCnt >= $self->{iloscProblemow} || $grpProblCnt >= $grpProblLimit) && !$wszystkieProblemy);
      # Ustawiamy wskanik do problemu
      my $selectedProblemInGroup=$grpProblIndexes[$grpProblCnt];
      my $problem=$problemsRef->{$selectedProblemInGroup};
      # Zwiekszamy licznik
      # Drukujemy tekst nagwkowy tego problemu
      $form.="\n\n\\pro  $problem->{text}";
      # Dopisujemy znak zmiany linii jeli kada odpowied ma by w nowej linii
      $form.="\\\\" if($self->{odpowiedziWNowejLinii});

      # Przechodzimy do odpowiedzi tego problemu
      my $answersRef=$problem->{answers};
      my @ansIndexes=sort keys(%$answersRef);
      @ansIndexes=randomPerm(@ansIndexes) if($numerTestu != 0);
      my $ansNmb=$#ansIndexes+1;
      $copyAnswerKey->[$problCnt]=[];
      $reverseNumberingKey->[$problCnt]=[];

      my @answers;
      for(my $ansCnt=0;$ansCnt<$ansNmb;$ansCnt++){
        my $selectedAnswer=$ansIndexes[$ansCnt];
        my $odpowiedz=$answersRef->{$selectedAnswer};
        # Drukujemy tekst odpowiedzi
        # generalnie bez zaznaczania poprawnych odpowiedzi,
        # chyba, e argument $rozwiazany jest niezerowy
        my $a;
        if($rozwiazany){
          $a=($odpowiedz->{value} ?  "\n\\ok $odpowiedz->{text}" : "\n\\odp $odpowiedz->{text}");
        }else{
          $a="\n\\ok $odpowiedz->{text}";
        }
        # Dopisujemy znak zmiany linii jeli kada odpowied ma by w nowej linii
        push(@answers,$a);
#        $form.="\\\\" if($self->{odpowiedziWNowejLinii});
        # Zapamitujemy odpowied w haszu odpowiedzi
        $copyAnswerKey->[$problCnt]->[$ansCnt]=$odpowiedz->{value};
        $reverseNumberingKey->[$problCnt]->[$ansCnt]="$selectedGroup:$selectedProblemInGroup:$selectedAnswer";
      }
      my $answersShort=join('',@answers);
      my $answersLong=join('\\\\',@answers);
      if(length($answersShort) < 60 || !$self->{odpowiedziWNowejLinii}){
        $form.=$answersShort;
      }else{
        $form.=$answersLong;
      }
      # Jesli odpowiedzi jest mniej ni maximum, extra odpowiedzi ustawiamy jako puste
      for(my $k=$ansNmb;$k<$self->{maxIloscOpcji};$k++){
        $copyAnswerKey->[$problCnt]->[$k]=0;
      }
      # Drukujemy tekst kocowy tego problemu
      if($problem->{closingText} =~/\S/){
        $form.="\\\\$problem->{closingText}";
      }
      if($self->{par}->{separatorProblemow}){
        $form.="\\vspace*{$self->{par}->{separatorProblemow}mm}\n";
      }
      $problCnt++;
      $grpProblCnt++;
    }
    $grpCnt++;
  }
  if($self->{par}->{wielokartkowy}){
    $form.="\\\\[3mm]{\\bf Nr testu:} \\framebox[14mm][l]{$numerEdycjiTerminu$numerGrupySal-$numerTestuSformatowany}\n";
  }
  $form.="$self->{testCopyTail}\n\n";
  $form.="\\newpage\n";
  return ($form,$copyAnswerKey,$reverseNumberingKey);
}

################################
#
# Funkcja przygotowuje plik HTML z rozszyfrowaniem odpowiedzi dla danego numeru testu
# Przygotowana na podstawie pliku writeHTMLstudentAnswers w examTestEvaluator
#
sub writeHtmlKeyForOneTest{
  my ($self,$numerTestu)=@_;

  my @numerTestuBin=toBin($numerTestu,9);
  my @numerTestuIcons=();
  my @numerTestuBinX=();
  for(my $i=0;$i<=$#numerTestuBin;$i++){
    $numerTestuIcons[$i]=( $numerTestuBin[$i] ? "_markOK.bmp" : "_unmarkOK.bmp" );
    $numerTestuBinX[$i]=( $numerTestuBin[$i] ? 'X' : '&nbsp;&nbsp;' );
  }

  my $text=<<END
<html>
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=windows-1250">
  </head>
  <body>
  <h2> Klucz testu: $numerTestu </h2>
  <table border=1>
    <tr> <td> $numerTestuBinX[0] <td> $numerTestuBinX[1] <td> $numerTestuBinX[2]
    <tr> <td> $numerTestuBinX[3] <td> $numerTestuBinX[4] <td> $numerTestuBinX[5]
    <tr> <td> $numerTestuBinX[6] <td> $numerTestuBinX[7] <td> $numerTestuBinX[8]
  </table>
  <div style="font-family:monospace">
  <h3> Wykaz odpowiedzi </h3>
  Klucz:<br>
  <font style="background:#ff8080">&nbsp;</font> - niepoprawna<br>
  <font style="background:#80ff80">X</font> - poprawna <br><br>
END
  ;
  my $correctAnswers=$self->{answers}->[$numerTestu];
#  my $reverseNumbering=$self->{reverseNumbering}->[$student->{nrTestu}];
  for (my $i = 0; $i < $self->{iloscProblemow}; $i++){
    my $pyt=fill_to_left($i + 1,6).") ";
    $pyt =~ s/ /&nbsp;/g;
    $text.=$pyt;
    for(my $j=0;$j<$self->{maxIloscOpcji};$j++){
      $text.=chr(ord("A") + $j).'&nbsp;';
      # Dwukrotne powtrzenie, bo oznaczamy tak jakby zosta wypeniony poprawnie
      # W wersji z writeHTMLstudentAnswers w examTestEvaluator na pierwszym miejscu
      # s odpowiedzi studenta
      my $scA=$correctAnswers->[$i]->[$j].$correctAnswers->[$i]->[$j];

      # A teraz wyznaczamy stosowny obrazek, a przy okazji robimy statystyk
      my $pict;
      if($scA =~ /0\s*0/){
         $pict='<font style="background:#ff8080">&nbsp;</font>';
      }elsif($scA =~ /0\s*1/){
         $pict='<font style="background:#80ff80">&nbsp;</font>';
      }elsif($scA =~ /1\s*0/){
         $pict='<font style="background:#ff8080">X</font>';
      }else{
         $pict='<font style="background:#80ff80">X</font>';
      }
      $text.="$pict &nbsp;&nbsp;";
    }
    $text.="<br><br>\n";
  }
  $text.="</div></body></html>\n";
  return $text;
}

################################
#
# Funkcja przygotowuje komplet testow
# i odpowiednie klucze odpowiedzi
#
sub generateCopies{
  my ($self,$answers)=@_;
  for(my $numerTestu=0;$numerTestu<$self->{iloscTestow};$numerTestu++){

    my($form,$copyAnswerKey,$reverseNumberingKey)=$self->generateCopy($numerTestu);
    # Zapamitujemy tekst wygenerowanego formularza testu
    $self->{tests}->[$numerTestu]=$form;
    # Zapamitujemy dane do odtworzenia oryginalnych numerw pyta i odpowiedzi
    $self->{reverseNumbering}->[$numerTestu]=$reverseNumberingKey;
    # Oraz stosowny klucz odpowiedzi
    $answers->[$numerTestu]=$copyAnswerKey;
  }
}

################################
#
# Funkcja zapisuje komplet testow
# oraz indywidualne testy jako oddzielne pliki (bez nagwkw)
#
sub writeCopies{
  my ($self,$answers)=@_;

  open(TESTS,">$self->{output}");
  print TESTS $self->{testCopiesHead};
  my $numerTestu;
  for($numerTestu=0;$numerTestu<$self->{iloscTestow};$numerTestu++){

#   Do pliku z testami wypisujemy tylko testy dla studentw,
#   natomiast pomijamy test zero, ktry zawiera wydrukowany klucz
    print TESTS $self->{tests}->[$numerTestu] if($numerTestu);
    open(ONE_TEST,">test-$numerTestu.tex");
    print ONE_TEST $self->{tests}->[$numerTestu];
    close(ONE_TEST);
    # Jeli wczono opcj generowania plikw HTML z odpowiedziami, generujemy je
    if($self->{par}->{generujPlikiOdpowiedzi}){
      open(ONE_HTML_KEY,">test-$numerTestu.html");
      print ONE_HTML_KEY $self->writeHtmlKeyForOneTest($numerTestu);
      close(ONE_HTML_KEY);
    }
  }
  print TESTS $self->{testCopiesTail};
  close(TESTS);
}

################################
#
# Funkcja zapisuje test numer zero
# z naniesionymi poprawnymi odpowiedziami
# i w celach kontrolnych bez zaznaczonych poprawnych odpowiedzi
#
sub writeSolvedTestAndControlTest{
  my ($self,$answers)=@_;

  # Generujemy plik komplentego testu (wszystkie pytania) z naniesionymi poprawnymi odpowiedziami
  open(TESTS,">$self->{solvedTest_output}");
  print TESTS $self->{testCopiesHead};
  my ($form,$copyAnswerKey,$reverseNumberingKey)=$self->generateCopy(0,1,1);
  print TESTS $form;
  print TESTS $self->{testCopiesTail};
  close(TESTS);

  # Generujemy plik testu kontrolnego bez naniesionych poprawnych odpowiedzi
  open(TESTS,">$self->{controlTest_output}");
  print TESTS $self->{testCopiesHead};
  my($form,$copyAnswerKey,$reverseNumberingKey)=$self->generateCopy(0,0);
  print TESTS $form;
  print TESTS $self->{testCopiesTail};
  close(TESTS);

}

################################
# Generujemy plik z kluczem odpowiedzi
# numery studentw s w pliku o rozszerzeniu *.sei
#
sub writeTestKey{
  my ($self,$answers)=@_;
  open(ANS,">$self->{testkey_output}") || $self->{raport}->zakoncz("Failed to open $self->{testkey_output}\n");
  print ANS "Seria: $self->{seria}\n";
  print ANS "iloscProblemow: $self->{iloscProblemow}\n";
  print ANS "maxIloscOpcji: $self->{maxIloscOpcji}\n";
  my ($i,$j,$k);
  for($i=1;$i<=$self->{numberOfSelectedGroups};$i++){
    my $gid="g".fill_with_zeros($i,4);
    my $gsize=$self->{groups}->{$gid}->{numberOfSelectedProblems};
    print ANS "grupa[$i]=$gsize \n";
  }
  # Zapisujemy klucze odpowiedzi
  for($i=0;$i<$self->{iloscTestow};$i++){
    my $test=fill_to_right($i,3);
    print ANS "test $test";
    for($j=0;$j<$self->{iloscProblemow};$j++){
      my $problem=fill_to_left($j+1,3);
      print ANS " :$problem)";
      for($k=0;$k<$self->{maxIloscOpcji};$k++){
        print ANS " $answers->[$i]->[$j]->[$k]";
      }
    }
    print ANS " \n";
  }
  # Zapisujemy informacj zwrotn pozwalajc odtworzy
  # oryginalny numer grupy, pytania w grupie oraz odpowiedzi
  for($i=0;$i<$self->{iloscTestow};$i++){
    my $test=fill_to_right($i,3);
    print ANS "testRN $test";
    for($j=0;$j<$self->{iloscProblemow};$j++){
      my $problem=fill_to_left($j+1,3);
      print ANS " |$problem)";
      for($k=0;$k<$self->{maxIloscOpcji};$k++){
        print ANS " $self->{reverseNumbering}->[$i]->[$j]->[$k]";
      }
    }
    print ANS " \n";
  }
  close(ANS);
}

################################
# Odczytujemy plik z kluczem
#
sub readTestKey{
#  my ($self,$answers)=@_;
  my ($self)=@_;
  open(ANS,"<$self->{testkey_output}") || $self->{raport}->zakoncz("Failed to open $self->{testkey_output}\n");
#  $self->{answers}=[] if(!$self->{answers});
#  $answers=$self->{answers};
  my $answers=[];
  $self->{reverseNumbering}=[] if(!$self->{reverseNumbering});
  while(<ANS>){
    chop();
    if(/^test\s+(\d+)\s*:(.*)$/){
      my ($i,$j);
      my $testNo=$1;
      my $a=$2;
      my @ans=split(/\s*:\s*/,$a);
      for($i=0;$i<=$#ans;$i++){
        my ($prob,$b)=split(/\)\s+/,$ans[$i]);
        my @probans=split(/\s+/,$b);
        for($j=0;$j<=$#probans;$j++){
          $answers->[$testNo]->[$i]->[$j]=$probans[$j];
        }
      }
    }elsif(/^testRN\s+(\d+)\s*\|(.*)$/){
      my ($i,$j);
      my $testNo=$1;
      my $a=$2;
      my @ans=split(/\s*\|\s*/,$a);
      for($i=0;$i<=$#ans;$i++){
        my ($prob,$b)=split(/\)\s+/,$ans[$i]);
        my @probans=split(/\s+/,$b);
        for($j=0;$j<=$#probans;$j++){
          $self->{reverseNumbering}->[$testNo]->[$i]->[$j]=$probans[$j];
        }
      }
    }elsif(/^Seria: (.*)$/){
      $self->{seria}=$1;
    }elsif(/^iloscProblemow: (.*)$/){
      $self->{iloscProblemow}=$1;
    }elsif(/^maxIloscOpcji: (.*)$/){
      $self->{maxIloscOpcji}=$1;
    }elsif(/^\s*grupa\[(\d+)\]\s*=\s*(\d+)\s*$/){
      my ($g,$w)=($1,$2);
      if(!defined($self->{'wielkoscGrup'})){
        $self->{'wielkoscGrup'}=[];
      }
      $self->{'wielkoscGrup'}->[$g]=$w;
    }
  }
  close(ANS);
  return $answers;
}

#######################################
# Testujemy czy istnieje plik z kluczem
#
sub checkTestKeyExistence{
  my ($self)=@_;
  my $ok=0;
  my $keyExists=0;
  if(-e "$self->{testkey_output}"){
    # Plik istnieje, sprawdzamy. czy zgadza si seria
    $keyExists=1;
    open(ANS,"<$self->{testkey_output}") || $self->{raport}->zakoncz("Failed to open $self->{testkey_output}\n");
    while(<ANS>){
      chop();
      if(/^Seria: (.*)$/){
        my $seria=$1;
        # W przeczytanej serii mog by stare daty z kropk w miejsce kreski
        $seria =~ s/\./-/g;
        $ok=1;
        # Seria jest inna, prosimy o potwierdzenie
        if($self->{seria} ne $seria){
          my $answer=$self->{raport}->{mainWindow}->Dialog(
             -title => 'Zmiana klucza',
             -text => "Istniej ju pliki z odpowiedziami i to o innym numerze serii:\n".
                      "$self->{seria} != $seria\n".
                      "Mogo si tak sta jeli zmieniono haso, nazw lub dat egzaminu\n".
                      "Jeli jeszcze nic nie wydrukowano, mona bezpiecznie odpowiedzie TAK\n".
                      "Czy kontynuowa?".
                      "Jeli odpowiesz TAK, pliki te zostan bezpowrotnie zniszczone.\n",
             -default_button => 'NIE',
             -buttons => ['TAK','NIE'],
           )->Show();
           if($answer ne 'TAK'){die "Przerwano w zwizku z grob zniszczenia klucza"};
        }
      }
      if(/^iloscProblemow:\s*(\d+)\s*$/){
        my $iloscProblemow=$1;
        if($self->{iloscProblemow} ne $iloscProblemow){
             $self->{raport}->wymaluj("Bdna ilo problemw w kluczu: usu plik auxil/*.ans \ni rozpocznij od nowa\n");
             die "Bdna ilo problemw w kluczu: usu plik auxil/*.ans \ni rozpocznij od nowa\n";
        }
      }
      if(/^maxIloscOpcji:\s*(\d+)\s*$/){
        my $maxIloscOpcji=$1;
        if($self->{maxIloscOpcji} ne $maxIloscOpcji){
             $self->{raport}->wymaluj("Bdna ilo opcji w kluczu: usu plik auxil/*.ans \ni rozpocznij od nowa\n");
             die "Bdna ilo opcji w kluczu: usu plik auxil/*.ans \ni rozpocznij od nowa\n";
        }
      }
    }
    close(ANS);
    if(!$ok){
      confirmContinuation("Istnieja juz pliki z odpowiedziami, ale brak w nich numeru serii.\nPliki te zostana bezpowrotnie zniszczone.\n");
    }
  }
  return $keyExists;
}

################################
#
# Generujemy pliki dvi
#
# zaszo
#
sub generateDvi{
  my ($self, $filename)=@_;
#  my $out=`latex $filename`;
  my $out=`$self->{callerRef}->{par}->{TexBin}latex $filename`;

}


################################
#
# Generujemy pliki postscript'owe
#
# zaszo
#
sub generatePs{
  my ($self, $filename, $psOptionalArguments)=@_;
  # Uwaga $filename przychodzi z rozszerzeniem tex
  $filename =~ s/tex$/dvi/;
#  my $out=`dvips $psOptionalArguments $filename -o`;
  my $out=`$self->{callerRef}->{par}->{TexBin}dvips $psOptionalArguments $filename -o`;
}


################################
#
# Generujemy pliki PDF
#
# zaszo
#
sub generatePDF{
  my ($self, $filename, $psOptionalArguments)=@_;
  my $out=`$self->{callerRef}->{par}->{TexBin}pdflatex --interaction=batchmode --src $filename`;
}


################################
# Wyznaczamy wyniki
#
sub evaluateStudentAnswers{
  my ($self)=@_;
  $self->{raport}->zakoncz("examTest:: evaluateStudentAnswers not implemented in abstract class\n");
}



################################
# Drukujemy wyniki
#
sub printTestEvaluation{
  my ($self)=@_;
  $self->{raport}->zakoncz("examTest:: printTestEvaluation not implemented in abstract class\n");
}



################################
#
# Wybr waciwego drukarza
#
sub selectPrinter{
  my ($self)=@_;
  my %listaRodzajowDrukarzy = (1 => "NONE",
                               2 => "DVI",
                               3 => "PS",
                               4 => "PDF");

  my $drukarz;
  if($self->{formatKoncowy} >= 2){
    my $typDrukarza="printer$listaRodzajowDrukarzy{$self->{formatKoncowy}}";
    require "$typDrukarza.pm";
    $drukarz=$typDrukarza->new($self);
  }else{
    $drukarz='none';
  }
  return $drukarz;
}

################################
#
# Przesuwamy do waciwych katalogw
#
sub relocateGeneratedFiles{
  my ($self)=@_;

  opendir (DIRHANDLE, '.');
  my @filelist = readdir (DIRHANDLE);
  closedir(DIRHANDLE);

#print "----->Relocating\n";
  # Pliki tex, dvi, ps, pdf przesuwamy do ich katalogw
  foreach my $file (@filelist){
    next if($file !~ /\.(\w+)$/);
    my $suff=$1;
    next unless($suff eq 'tex' || $suff eq 'dvi' || $suff eq 'ps' || $suff eq 'pdf' || $suff eq 'html');
    if(-d $suff){
#print "    Relocating $file\n";
      my $toRemove="$suff/$file";
      unlink <$toRemove>;
      move($file,$suff);
    }
  }
  # Usuwamy pliki aux i log
  unlink <*.aux>;
  unlink <*.log>;

}

################################
#
# Przygotowywujemy testy
#
sub differenceInAnswers{
  my ($self,$answers1,$answers2)=@_;
  for(my $i=0;$i<$self->{iloscProblemow};$i++){
    for(my $j=0;$j<$self->{maxIloscOpcji};$j++){
      # Klucz jest przechowywany pod indeksem 0
      if($answers1->[0]->[$i]->[$j] != $answers2->[0]->[$i]->[$j]){
        return 1;
      }
    }
  }
  return 0;
}

################################
#
# Przygotowywujemy testy
#
sub prepareTests{
  my ($self)=@_;
#  my ($self,$form,$studentsByExamId)=@_; # Pozostae argumenty nie s uywane
  my $answers=$self->{answers}=[];


  $self->read();
  $self->make_stamp();

  my $keyExists=$self->checkTestKeyExistence();
  $self->redoGroupsDescriptions() if($self->{par}->{oddzieloneOpisyGrup});
  my $drukarz=$self->selectPrinter();

  # Generujemy jeli nie jest wczona  opcja tylkoTestKontrolny
  if(!$self->{tylkoTestKontrolny}){
    $self->{raport}->wymaluj("Generuj testy\n");
    $self->generateCopies($answers);
    if($keyExists){
      # Sprawdzamy, czy klucz w istniejcym pliku jest ten sam
      my $answersFromFile=$self->readTestKey();
      if($self->differenceInAnswers($answers,$answersFromFile)){
          my $answer=$self->{raport}->{mainWindow}->Dialog(
             -title => 'Zmiana klucza',
             -text => "Wygenerowany wczeniej plik *.ans zawiera inny klucz odpowiedzi\n".
                      "Czy kontynuowa?\n".
                      "Jeli odpowiesz TAK, naley na nowo wygenerowa formularze odpowiedzi.\n".
                      "a ewentualne istniejce wydruki zniszczy.\n",
             -default_button => 'NIE',
             -buttons => ['TAK','NIE'],
           )->Show();
           if($answer ne 'TAK'){
             $self->{raport}->wymaluj("Przerwano w zwizku z grob zniszczenia istniejcego klucza\n");
             die "Przerwano w zwizku z grob zniszczenia istniejcego klucza"
           };
      }
    }
    $self->writeTestKey($answers);
    $self->{raport}->wymaluj("Generuj pliki TeXowe testw\n");
    $self->writeCopies($answers);
    $drukarz->printFile($self->{output}) if($self->{formatKoncowy} >= 2);
  }




  if($self->{formatKoncowy} >= 2){
    # Generujemy test kontrolny
    $self->writeSolvedTestAndControlTest($answers);
    $self->{raport}->wymaluj("Generuj test kontrolny i test kontrolny z kluczem\n");
    $drukarz->printFile($self->{solvedTest_output});
    $drukarz->printFile($self->{controlTest_output});
    if(  $self->{par}->{oddzieloneOpisyGrup} == 1){
       $drukarz->printFile($self->{groupDescriptions_output});
    }
  }
#print "--- self->{par}->{rozdzielonePliki}=$self->{par}->{rozdzielonePliki}=\n";
  $self->relocateGeneratedFiles() if($self->{par}->{rozdzielonePliki});
}

################################
#
# Przygotowywujemy formularze
#
sub prepareForms{
  my ($self,$form,$studentsByExamId)=@_;

#  $self->readTestKey($self->{answers}) if(!defined($self->{answers}->[0]));
  $self->{answers}=$self->readTestKey() if(!defined($self->{answers}->[0]));

  # Formularze odpowiedzi i niezbdnik wykadowcy
  # przygotowywujemy tylko gdy wczytana zostaa lista
  # studentw
  if($studentsByExamId){
    $self->{raport}->wymaluj("Generuj pliki TeXowe formularzy\n");
    $form->writeCopies($self,$studentsByExamId,$self->{answers}) ;
  }

  if(  $self->{formularzWzorcowy} == 1){ $form->writeTemplate($self)};

  my $drukarz=$self->selectPrinter();
  if($self->{formatKoncowy} >= 2){
    if($studentsByExamId){
      $drukarz->printFile($self->{forms_output});
      if( $self->{formularzWzorcowy} == 1 ){
        $drukarz->printFile($self->{template_output});
      }
    }
  }
  $self->relocateGeneratedFiles();
}

################################
#
# Przygotowywujemy testy i formularze
# Uywane tylko w starej wersji pracujcej w okienku DOS
#
sub prepareTestsAndForms{
  my ($self,$form,$studentsByExamId)=@_;

  $self->prepareTests();
#  $self->prepareTests($form,$studentsByExamId);  # Pozostae argumenty nie s uywane
  $self->prepareForms($form,$studentsByExamId);
}

###########################################################################

1;
