################################
#
# klasa abstrakcyjna robAutoTest
#
# Pochodna od examTest2XML
#
# Przeznaczona do przerabiania testw
# egzaminowych w TeXu na testy
# do samodzielnego sprawdzania si w HTMLu

package robAutoTest;



use utilities;
use examTest2XML;
@ISA = ("examTest2XML");

################################
#
# Funkcja zamyka tekst w CDATA ( w examTest2XML )
#
# nie jestem pewien czy chc usuwa sekcje CDATA, ale na razie software
# Przybycienia nie dziaa tak jak chc na sekcjach CDATA:

sub cdata{
  my ($self,$text)=@_;
  return $text;
}



################################
#
# Funkcja zmienia formatowanie TeXowe na HTMLowe
#
sub convertTeX2HTML{
  my ($self,$text)=@_;
  $text =~ s/\{?\s*\$\^\{\s*\\wedge\s*\}\s*\$\s*\}?/^/g;        # \wedge
  $text =~ s/\$<\$/&lt;/g;                                      #  $<$
  $text =~ s/\$>\$/&gt;/g;                                      #  $>$
  $text =~ s/\$<<\$/&lt;&lt;/g;                                 #  $<<$
  $text =~ s/\$>>\$/&gt;&gt;/g;                                 #  $>>$
  $text =~ s/</&lt;/g;                                          #  <
  $text =~ s/>/&gt;/g;                                          #  >
  $text =~ s/\\begin{verbatim}/<pre>/g;                         #  \begin{verbatim}
  $text =~ s/\\end{verbatim}/<\/pre>/g;                         #  \end{verbatim}
  $text =~ s/\\tight\s*\n?//g;                                  # \tight
  $text =~ s/\{\\bf\s+([^}]+)\s*\}/<b> $1 <\/b>/g;              # {\bf }
  $text =~ s/<!\[CDATA\[//g;                                    # <[CDATA[
  $text =~ s/]]>//g;                                            # ]]>
  $text =~ s/\\\$/\$/g;                                         # \$
  $text =~ s/\\&/&amp;/g;                                       # \&
  $text =~ s/\\_/_/g;                                           # \_
  $text =~ s/\{\\tt\s+([^}]+)\s*\}/<b class=code> $1 <\/b>/g;   # {\tt }
  $text =~ s/\{\\small\s+(.*)\}/$1/g;                           # {\small  }
  return $text;
}

################################
#
# Funkcja zapisuje test z przetasowanymi odpowiedziami w formacie XML
#
sub XMLformMixed{
  my ($self,$totalProblemCounterRef)=@_;
  my $out="<?xml version=\"1.0\" encoding=\"windows-1250\"?>\n";
  $out.="<test>\n";
  my $indentUnit=4;
  my $currentIndent=0;
  foreach my $groupId (sort keys(%{$self->{groups}})){
    $currentIndent+=$indentUnit;
    my $group=$self->{groups}->{$groupId};
    $out.=$self->blanks($currentIndent)."<group>\n";
    $indent=$currentIndent;
    $group->{text}=$self->convertTeX2HTML($group->{text});
    $out.=$self->makeIndent($group->{text},$self->blanks($currentIndent+$indentUnit));
    foreach my $problemId (sort keys(%{$group->{problems}})){
      $$totalProblemCounterRef++;
      $currentIndent+=$indentUnit;
      my $problem=$group->{problems}->{$problemId};
      $out.=$self->blanks($currentIndent)."<problem>\n";
      $out.=$self->makeIndent($self->convertTeX2HTML($problem->{text}),$self->blanks($currentIndent+$indentUnit));
      my @indexes=sort keys(%{$problem->{answers}});
      @indexes=randomPerm(@indexes);
      foreach my $answerId (@indexes){
        $currentIndent+=$indentUnit;
        my $answer=$problem->{answers}->{$answerId};
        if($answer->{value}){
          $out.=$self->blanks($currentIndent)."<answer value=\"true\" >\n";
        }else{
          $out.=$self->blanks($currentIndent)."<answer value=\"false\" >\n";
        }
        $out.=$self->makeIndent($self->convertTeX2HTML($answer->{text}),$self->blanks($currentIndent+$indentUnit));
        $out.=$self->blanks($currentIndent)."</answer>\n";
        $currentIndent-=$indentUnit;
      }
      $out.=$self->blanks($currentIndent)."</problem>\n";
      $currentIndent-=$indentUnit;
    }
    $out.=$self->blanks($currentIndent)."</group>\n";
    $currentIndent-=$indentUnit;
  }
  $out.="</test>\n";
  return $out;
}


################################

1;
