##################
#
# klasa rekord
#
# klasa obsuguje we-wy haszowych rekordw
#

package Record;
require Tie::Hash;

@ISA =(Tie::Hash);

use vars qw($AUTOLOAD);
use vars qw($error);
use strict;
use utilities;

$error=0;



################################
#
# odgaduje separator
#
sub guessSep{
  my ($list,$sep) = @_;
  $list=eject_blanks($list);
  if(!defined($sep) || $sep eq ''){
    if($list=~/\t/){
      $sep="\t" ;
    }elsif($list=~/;/){
      $sep=";" ;
    }elsif($list=~/,/){
      $sep="," ;
    }else{
      $sep="\\s+";
    }
  }
  return $sep;
}

################################
#
# Konstruktor
#
sub new{
  my $class=shift;
  my $recordClass=shift;
  my $self = {};
  $self->{fieldNames}=$recordClass->fields;
  bless($self,$class);
  return $self;
}

################################
#
# Konstruktor
#
#!!!!!!! Najprawdopodobniej wymaga zmian po ostatnich przerobkach
sub newFromList{
  my $class=shift;
  my $fieldsList=shift;
  my ($sep) = @_;
  $fieldsList=eject_blanks($fieldsList);
  $sep=guessSep($fieldsList,$sep);
  my @keys=split($sep,$fieldsList);
  return $class->new(\@keys);
}

################################
#
# Przypisuje kolejnym kluczom kolejne wartoci z listy wg
# porzadku kluczy podanego w tablicy wskazywanej przez $keyOrderRef
# Jesli argumentu tego nie podano przyjmuje sie porzadek kluczy rekordu
#
sub set{
  my ($self,$list,$fieldsOrderArrayRef,$sep) = @_;
#print "Got $list\n";
  $sep=guessSep($list,$sep);
  $fieldsOrderArrayRef=$self->{fieldNames} if(!ref($fieldsOrderArrayRef));
  my @value=split($sep,$list);
  my $length=$self->length;
  for(my $i=0;$i<$length;$i++){
    my $val=eject_blanks($value[$i]);
    # Korekta liczb floating point
    $val =~ s/^\s*(\d*),(\d+)\s*$/$1.$2/;
    if($fieldsOrderArrayRef->[$i] =~/\S/ && defined($self->{$fieldsOrderArrayRef->[$i]})){
      print "Field $fieldsOrderArrayRef->[$i] already defined.\n  Check synonims in StudentRecordBase.pm\n";
    }else{
      $val=eject_blanks($val);
#print "Setting $fieldsOrderArrayRef->[$i] to =$val=\n";
      $self->{$fieldsOrderArrayRef->[$i]}=$val;
    }
  }
}

################################
#
# funkcja zwraca rekord jako napis
#
sub toString{
  my ($self,$fieldsList,$sep) = @_;
  my $gsep=guessSep($fieldsList,'');
  $sep=';' if(!defined($sep));
  my @pole=split($gsep,$fieldsList);
  my $string='';
  my $length=$self->length;
  for(my $i=0;$i<$length;$i++){
    my $field=$self->{$self->{fieldNames}->[$i]};
    if($pole[$i]>0){
      $field=fill_to_right($field,$pole[$i]);
    }elsif($pole[$i]<0){
      $field=fill_to_left($field,-$pole[$i]);
    }
    $string.="$field$sep";
  }
  return $string;
}

################################
#
# funkcja zwraca klucz identyfikujacy rekord
#
sub key{
  my ($self) = @_;
  return $self->Nazwisko;
}

################################
#
# metoda sprawdza istnienie pola
#
sub exists{
  my ($class,$field)=@_;
  my $found=0;
  my $length=$class->length;
  search: for (my $i=0;$i<=$length;$i++){
    if($class->fields->[$i] eq $field){
#my $t=$class->fields->[$i];
#print "Trying =$i= =$t= =$field=\n";
      $found=1;
      last search;
    };
  }
#print "Ret =$found=\n";
  return $found;
}

################################
#
# metoda dodaje dodatkowe pola z tablicy
#
sub addFields{
  my $class=shift;
  my $fieldsArrayRef=shift;
  foreach my $key (@$fieldsArrayRef){
    $key=eject_blanks($key);
    if(!$class->exists($key)){
      push(@{$class->fields},$key)
    }
  }
}

################################
#
# metoda dodaje dodatkowe pola z listy
#
sub addFieldsFromList{
  my $class=shift;
  my $fieldsList=shift;
  my ($sep) = @_;
  $fieldsList=eject_blanks($fieldsList);
  $sep=guessSep($fieldsList,$sep);
  my @keys=split($sep,$fieldsList);
  $class->addFields(\@keys);
}

################################
#
# funkcja obsuguje nazwy pl
#
sub AUTOLOAD{
  my $self=shift;
  my $field=$AUTOLOAD;
#print "Autoload called with =$self= =$field=\n";
  $field =~ s/.*:://;
  return unless $field =~ /[^A-Z]/;
  die("Unrecognized field $field\n") if(!$self->exists($field));
  my ($value)=@_;
  if(defined($value)){
#print "Setting value =$value=\n";
    $self->{$field}=$value;
  }
#print "Returning value =$self->{$field}=\n";
  return $self->{$field};
}

################################
#
# Dolacza do pustych pol rekordu dane z drugiego rekordu
# Zglasza blad, jesli niepuste dane sa rozne
# UWAGA: klasa student ma wasn wersj
#
sub merge{
  my ($self,$merged_record) = @_;
  my $length=$self->length;
  for(my $i=0;$i<$length;$i++){
    my $field=$self->{fieldNames}->[$i];
    my $merged_record_field=$merged_record->{$field};

    # Korekta liczb floating point
    $merged_record_field =~ s/^\s*(\d*),(\d+)\s*$/$1.$2/;

    if(defined($self->{$field}) && $self->{$field} ne "" ){
       if(defined($merged_record_field) && $merged_record_field ne "" ){
         if(($self->{$field} ne $merged_record_field) && $field ne 'liczbaPorzadkowa'){
            my $key=$self->key;
            print "Merging field $field for key =$key= with unequal nontrivial values: $self->{$field} and $merged_record_field\n";
            $error=1;
         }
       }
    }else{
       if(defined($merged_record_field)){
          $self->{$field}=$merged_record_field;
       }
    }
  }
}

################################
#
# Pozostawia w rekordzie tylko pola, ktre s we wzorcowym rekordzie
#

sub leaveOnly{
  my ($self,$masterRecord) = @_;
  my $length=$self->length;
  for(my $i=0;$i<$length;$i++){
    my $field=$self->{fieldNames}->[$i];
    if(!defined($masterRecord->{$field})){
      delete($self->{$field}) ;
    }
  }
}

################################
#
# Usuwa z rekordu te pola, ktre s we wzorcowym rekordzie
#

sub removeFields{
  my ($self,$masterRecord) = @_;
  my $length=$self->length;
  for(my $i=0;$i<$length;$i++){
    my $field=$self->{fieldNames}->[$i];
    if(defined($masterRecord->{$field})){
      delete($self->{$field}) ;
    }
  }
}

################################

1;