##################
#
# klasa RecordBase
#
# klasa obsluguje baze rekordow klasy Record.pm
#
# Jedynymi kluczami w haszu obiektu tej klasy
# s klucze tworzone dla poszczegolnych rekordw
# metod key, ktr ma obowizek dostarczy metoda klasy pochodnej
# od klasy Record.pm
#
# Zasad t naley zachowa przy tworzeniu ewentualnych
# klas pochodnych tej klasy
#

package RecordBase;

use FileHandle;
use Record;
use polChar;
use utilities;
use textUtil;
use vars qw($warnOnFindingNewRecords $warnOnFindingKeyRepetitions);

$warnOnFindingNewRecords=1;

################################
#
# Konstruktor
#
sub new{
  my ($class) = @_;
  my $self = {};
  bless($self,$class);
  return $self;
}


################################
#
# Konstruktor, wybiera podane pola z wzorca i wedlug niego tworzy nowa liste
#
sub select{
  my ($self,$field,$value) = @_;
  $value=eject_blanks($value);
  my $newRecordBaseRef = {};
  bless($newRecordBaseRef,ref($self));
  while(my($recordKey,$recordRef)=each(%$self)){
     my $record_value=eject_blanks($recordRef->{$field});
     if( $record_value eq $value){
       $newRecordBaseRef->{$recordRef->key}=$recordRef;
     }
  }
  return $newRecordBaseRef;
}


#################################################
# wczytanie parametrow z naglowka pliku z ocenami
# (nazwa kursu, wykladowca, itp)
#
sub readHeading{
  my ($self,$fh,$parLoc)=@_;
  loop:
  while($_=$fh->getline){
    $_=remove_quotations($_);
    next if(/\[Nagwek\]/);
    next if(/\[Naglowek\]/);
    next if(/\[Head\]/);
    next if(/^\s*[\#;]/);
    last if(/^\s*$/);
    last if(/^\s*\[List/);
    chop($_);
    my ($sep,$name,$value);
    if(/^([^;=\t]*)(\s*[;=\t]\s*)(.*)$/){
      ($name,$sep,$value)=($1,$2,$3);
    }else{
       next;
    }
    $name =~ s/ //g;
    # Excel czasem produkuje pola w cudzysowach. Trzeba je zdj.
    if($value =~ /^\s*\"?(.*?)\"?\s*$/){ #"
      $value=$1;
    }
    $name=kapitalizujNormalizujKlucz($name);
#    $parLoc->{$name} = $value;
    # Zapisujemy warto, ale tylko nietrywialn, chyba, e
    # warto jest jeszcze nie okrelona
    $parLoc->{$name} = $value if($value =~ /\S/ || !defined($parLoc->{$name}));
#    print "Found $name$sep$value\n" if($main::opt_v);
  }
}

################################
#
# Funkcja czyta rekordy
#
sub readRecords{
  my ($self,$fh,$recordClass,$fieldsOrderArrayRef)=@_;
  my $line;
  loop:
  while(!$fh->eof){
    my $x=$fh->getline;
    $x=polUTF8_2win($x);
#print "Got line =$x=\n";
    $_=remove_quotations($x);
    next if(/^\s*[\#]/);
    next if(/^\s*$/);
    s/\n//;
    #   Tworzymy nowy rekord w oparciu o klucze zawarte w klasie rekordu
    my $record=new Record($recordClass);
    bless($record,$recordClass);
    #   Ustawiamy wartoci pl wg kolejnoci podanej w tablicy wyznaczajacej ich porzadek
    $record->set($_,$fieldsOrderArrayRef);
    my $key=$record->key;
    if(!defined($self->{$key})){
      $self->{$key}=$record;
    }else{
      print "Key $key already defined\n" if($warnOnFindingKeyRepetitions);
    }
    print "Read record $key \n" if($main::opt_v);
# print "****Read record $key \n";
  }
}

################################
#
# Funkcja dodaje/czy rekordy
#
sub mergeRecords{
  my ($self,$fh,$recordClass,$fieldsOrderArrayRef,$masterRecord,$fieldsToRemove)=@_;
  my $line;
  my $counter=0;
  loop:
  while(!$fh->eof){
    my $x=$fh->getline;
    $x=polUTF8_2win($x);
#print "Got line =$x=\n";
    $_=remove_quotations($x);
# print "Got $_";
    next if(/^\s*[\#]/);
    next if(/^\s*$/);
    s/\n//;
    ++$counter;
    my $record=new Record($recordClass);
    bless($record,$recordClass);
# print "Setting $_\n";
    $record->set($_,$fieldsOrderArrayRef);
    my $key=$record->key;

    $record->leaveOnly($masterRecord) if($masterRecord);
    $record->removeFields($fieldsToRemove) if($fieldsToRemove);

    my @fields=keys(%$record);
    if(defined($self->{$key})){
      $self->{$key}->merge($record);
# print "Merging: key=$key= from $_. Adding.\n";
    }else{
      $self->{$key}=$record;
      print "Found new record: key=$key= from $_. Adding.\n" if($warnOnFindingNewRecords);
    }
  }
  print "No records found!\n" if(!$counter);
}

################################
#
# Funkcja dodaje dane do istniejcych rekordw
# Ignoruje rekordy nie istniejce w bazie mergujcej
#
sub mergeRecordsData{
  my ($self,$fh,$recordClass,$fieldsOrderArrayRef,$silent,$masterRecord)=@_;
  my $line;
  loop:
  while(!$fh->eof){
    $_=remove_quotations($fh->getline);
    next if(/^\s*[\#]/);
    next if(/^\s*$/);
    s/\n//;
    my $record=new Record($recordClass);
    bless($record,$recordClass);
    $record->set($_,$fieldsOrderArrayRef);
    my $key=$record->key;
    $record->leaveOnly($masterRecord) if($masterRecord);
    if(defined($self->{$key})){
      $self->{$key}->merge($record);
       print "Merging data for record $key \n" if($main::opt_v);
    }elsif(!$silent){
       print "Warning: record: key=$key does not exist in the source base. Skipping\n";
    }
  }
}

################################
#
# Funkcja czyta baze z naglowkiem i rekordami
#
sub read{
  my ($self,$fileName,$par,$recordClass)=@_;
  my $fh=new FileHandle;
  $fh->open("<$fileName") or die("Failed to open $fileName");
  $self->readHeading($fh,$par);
  my $fieldsList=eject_blanks($par->{DaneRekordu});
  my $sep=Record::guessSep($fieldsList,$sep);
  my @fieldsOrderArray=split($sep,$fieldsList);
  #   Na wszelki wypadek mergujemy klucze zawarte w tablicy wyznaczajacej ich porzadek
  $recordClass->addFields(\@fieldsOrderArray);
  $self->readRecords($fh,$recordClass,\@fieldsOrderArray);
}

################################
#
# Funkcja czyta baze z naglowkiem i rekordami i czy z istniejc baz
#
sub merge{
  my ($self,$fileName,$par,$recordClass)=@_;
  my $fh=new FileHandle;
  $fh->open("<$fileName") or die("Failed to open $fileName");
  $self->readHeading($fh,$par);
  if(!$par->{DaneRekordu}){
    die("Niepoprawny format danych: ni znaleziono pola DaneRekordu w nagwku!");
  }
  my $fieldsList=eject_blanks($par->{DaneRekordu});
  my $sep=Record::guessSep($fieldsList,$sep);
  my @fieldsOrderArray=split($sep,$fieldsList);
  #   Na wszelki wypadek mergujemy klucze zawarte w tablicy wyznaczajacej ich porzadek
  $recordClass->addFields(\@fieldsOrderArray);
  $self->mergeRecords($fh,$recordClass,\@fieldsOrderArray);
}

################################
#
# Funkcja czyta baze z naglowkiem i rekordami i dodaje dane do istniejcyh ju rekordw
#
sub mergeData{
  my ($self,$fileName,$parLoc,$recordClass,$silent,$masterRecord)=@_;
  my $fh=new FileHandle;
  $fh->open("<$fileName") or die("Failed to open $fileName");
  # Jeli $parLoc ma nie okrelon warto, inicjujemy jako referencj do anonimowego hasza
  $parLoc={} if( !defined($parLoc) || $parLoc eq'');
  $self->readHeading($fh,$parLoc);
  my $fieldsList=eject_blanks($parLoc->{DaneRekordu});
  my $sep=Record::guessSep($fieldsList,$sep);
  my @fieldsOrderArray=split($sep,$fieldsList);
  #   Na wszelki wypadek mergujemy klucze zawarte w tablicy wyznaczajacej ich porzadek
  $recordClass->addFields(\@fieldsOrderArray);
  $self->mergeRecordsData($fh,$recordClass,\@fieldsOrderArray,$silent,$masterRecord);
}

################################
#
# Funkcja wypisuje plik z rekordami
#
sub write{
  # opis argumentw:
  # fileName              - nazwa pliku, do ktrego drukujemy
  # par                   - referencja hasza z parametrami
  # parFieldsArrayRef     - referencja tablicy z wykazem parametrow listy do wydrukowania
  # fieldsOrderArrayRef   - referencja tablicy z wykazem pol rekordu  do wydrukowania
  # sortingFieldsArrayRef - referencja tablicy z wykazem pol rekordu  do sortowania rekordow
  #    argumenty opcjonalne
  #
  # fieldsNamesArrayRef   - referencja tablicy z wykazem pol rekordu do wydrukowania (jeli chcemy zmieni im nazwy)
  # parFieldsNamesHashRef - referencja tablicy z nazwami parametrow listy do wydrukowania (jeli chcemy zmieni im nazwy)
  my $self=shift;
  my $fileName=shift;
  my $par=shift;
  my $parFieldsArrayRef=shift;
  my $fieldsOrderArrayRef=shift;
  my $sortingFieldsArrayRef=shift;
  my $fieldsNamesArrayRef=shift;
  $fieldsNamesArrayRef=$fieldsOrderArrayRef if(!defined($fieldsNamesArrayRef));
  my $parFieldsNamesHashRef=shift;

  my $fh=new FileHandle;
  $fh->open(">$fileName") or die("Failed to open $fileName");

  # Drukowanie nagwka
  print $fh "[Naglowek]\n";
  # chwilowo przechowujemy parametr DaneRekordu w zmiennej
  my $fieldsList=$par->{'DaneRekordu'};
  $par->{'DaneRekordu'}=join(";",@$fieldsNamesArrayRef);
  foreach my $elem (@$parFieldsArrayRef){
    my $elemName = (defined($parFieldsNamesHashRef->{$elem}) ? $parFieldsNamesHashRef->{$elem} : kapitalizujNormalizujKlucz($elem));
    print $fh "$elemName=$par->{$elem}\n";
  }
  $par->{'DaneRekordu'}=$fieldsList;

  # Szykujemy posortowana liste studentow
  my @toSort=();
  while(my($recordKey,$recordRef)=each(%$self)){
    my $sortkey='';
    foreach my $field (@$sortingFieldsArrayRef){
       $sortkey.="$recordRef->{$field} ";
    }
    push @toSort,"$sortkey***$recordKey";
  }

  # wypisujemy informacje o studentach wg kolejnosci ustalonej sortowaniem
  print $fh "[ListaStudentow]\n";
  my $sortedAdr=polSort(\@toSort);
  my @sorted=@$sortedAdr;
  my $lp=0;
  foreach my $elem (@sorted){
    my ($sortkey,$recordKey)=split(/\*\*\*/,$elem);
    my $recordRef=$self->{$recordKey};
    $recordRef->{liczbaPorzadkowa}=++$lp;
    foreach my $field (@$fieldsOrderArrayRef){
       print $fh "$recordRef->{$field};";
    }
    print $fh "\n";
  }
  $fh->close();
}



1;
