###############################
#
# Klasa konfigurator
#
# (c) Marian Mrozek 2002
#
#  Klasa umoliwia interaktywn aktualizacj
#  plikw konfiguracyjnych zapisanych w formacie
#  klucz=warto
#  wykorzystujcych znak # do opisu komentarzy

# Uwaga: Obecne rozwizanie jest konglomeratem dwch:
# 1) Konfiguracja poprzez przewijanie wszystkich parametrw
# w dolnym oknie tablicy
# 2) Konfiguracja poprzez wpis na list wybranych parametrw w grnym oknie
#
# W piewrszym rozwizaniu w tablicach
#  $self->{keysRef}
#  $self->{valuesRef}
# przechowywane s dane klucz warto uporzdkowane wedug ich kolejnoci
# w pliku conf
#
# W drugim (dodanym) rozwizaniu pary klucz, warto przechowywane s w haszu
#   $self->{keysValsHashRef}
#
# Zawarto tablic/haszy jest synchronizowana w metodach set/get

package konfigurator;

use Tk;
use strict;
use utilities;
require Tk::ROText;

################################
#
# Konstruktor
#

sub new{
  my $class = shift;
  my $self = {};
  bless($self,$class);
  (
    $self->{mainWindow},$self->{file},$self->{iniFile},$self->{par},$self->{caller},$self->{topDataFieldWidth},$self->{bottomDataFieldWidth}
  ) = @_;
  $self->{topDataFieldWidth}=30 if(!$self->{topDataFieldWidth});
  $self->{bottomDataFieldWidth}=80 if(!$self->{bottomDataFieldWidth});
  $self->init();
  $self->load();

  return $self;
}


###########################################################################
#
# Tworzy gwne okno konfiguratora i wypenia je pocztkow treci
#

sub configWindow{
  my $self = shift;

  #Tworzymy gwne okno
  $self->{ramka} = $self->{mainWindow}->Toplevel();
  $self->{ramka}->minsize(500,400);
  $self->{ramka}->title("Konfigurator");

  # Tworzymy potrzebne widgety
  my $f0 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'both');
  my $f1 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'both',, -expand => 1);
  my $f2 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'both');
  my $f3 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'x');
  my $f4 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'x');


  my $opis=<<END
Wybrane parametry mona modyfikowa na poniszej licie. Wszystkie parametry mona przeglda wraz z opisem
i modyfikowa w dolnym okienku. Parametru widocznego w dolnym okienku nie da si modyfikowa na licie.
Odfajkowana opcja "poka na licie" aktywowana jest dopiero po zapisaniu parametrw, przy kolejnym uruchomieniu konfiguratora.
END
  ;
  $f0->Label(-textvariable => \$opis, -anchor => 'c')->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  my @fParam;
  $self->{paramEntry}=[];
  for(my $i=0;$i<@{$self->{keysRef}};++$i){
    if($self->{visiblesRef}->[$i]){
      my $key= $self->{keysRef}->[$i];
      if($key ne 'noKey'){
        $fParam[$i] = $f1->Frame->pack(-side => 'top', -fill => 'both');
        $self->{paramEntry}->[$i]=$fParam[$i]->Entry(-textvariable => \$self->{keysValsHashRef}->{$key}, -width => $self->{topDataFieldWidth})->
          pack(-side => 'left', -anchor => 'e');
        $fParam[$i]->Label(-text => $key, -anchor => 'c')->
            pack(-side => 'left', -anchor => 'e');
      }
    }
  }

  $self->{descriptionWidget}=$f2->Scrolled('ROText',-scrollbars => 'osoe', -width => $self->{bottomDataFieldWidth}, -height => 5, -relief => 'ridge', -background => 'gray')->
    pack(-side => 'left', -anchor => 'e', -fill => 'both', -expand => 1);

  $f3->Label(-textvariable => \$self->{key}, -anchor => 'c')->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f3->Entry(-textvariable => \$self->{value})->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f3->Checkbutton(-text => 'poka na licie',-variable => \$self->{alwaysShowing}, -anchor => 'c')->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f4->Button(-text => "<< Poprzedni parametr", -command => ['go_left' => $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f4->Button(-text => "Nastpny parametr >>", -command => ['go_right' => $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f4->Button(-text => "Zapisz", -command => ['zapisz' => $self] )->
    pack(-side => 'left', -fill => 'x', -expand => 1);

  $f4->Button(-text => "Zakocz", -command => ['zakoncz' => $self] )->
    pack(-side => 'left', -fill => 'x', -expand => 1);

  $f4->Button(-text => "Zapisz i zakocz", -command => ['done' => $self] )->
    pack(-side => 'left', -fill => 'x', -expand => 1);

  # I przechodzimy do wywietlenia pierwszego parametru
  $self->go_right();
}

################################
#
# inicjalizacja
#

sub init{
  my $self = shift;

  $self->{posRef}={};
  $self->{keysRef}=[];
  $self->{valuesRef}=[];
  $self->{commentsRef}=[];
  $self->{visiblesRef}=[];
  $self->{keysValsHashRef}={}
}


###########################################################################
#
# ustawienie pocztkowe
#

sub reset{
  my $self = shift;
  $self->{ostatniaPozycja}=$#{@{$self->{keysRef}}};
  $self->{pozycja}=0;
  $self->{key}="xx";
  $self->{value}="yy";
  $self->{alwaysShowing}=0;
  $self->{showAll}=1;
  $self->set();
}


###########################################################################
#
# obsuga przycisku 'Zakocz'
#

sub zakoncz{
  my $self = shift;
  ${$self->{par}->{referencjaDoKonfiguratora}}=undef;
  $self->{ramka}->destroy();
  $self->{caller}->{egzMenu}->configure(-state => 'active') if($self->{caller});
}

###########################################################################
#
# obsuga przycisku 'Zapisz'
#

sub zapisz{
  my $self = shift;
  $self->get();
  $self->save();
  # Nowa konfiguracja zostaa zapisana do pliku
  # Trzeba j wczyta ponownie, by kolejne
  # zadania mogy z niej korzysta
  # Ostatni argument wymusza ponowne przeczytanie danych
  readConfiguration('.',1,$self->{par},"",1);
}

###########################################################################
#
# obsuga przycisku 'Zapisz i Zakoncz'
#

sub done{
  my $self = shift;
#  $self->get();
#  $self->save();
#  # Nowa konfiguracja zostaa zapisana do pliku
#  # Trzeba j wczyta ponownie, by kolejne
#  # zadania mogy z niej korzysta
#  # Ostatni argument wymusza ponowne przeczytanie danych
#  readConfiguration('.',1,$self->{par},"",1);
  $self->zapisz();
  $self->zakoncz();
}

###########################################################################
#
# Ustawienie aktualnych wywietlanych wartoci na podstawie konfigurowanej
# tablicy
#

sub set{
  my $self = shift;
  # Kolejny klucz pobieramy z tablicy
  $self->{key}=$self->{keysRef}->[$self->{pozycja}];
  # Ale jego wartos pobieramy z hasza wedug kluczy,
  # bo moga w midzy czasie si zmieni w grnum oknie
  $self->{value}=$self->{keysValsHashRef}->{$self->{key}};
  # I na wszelki wypadek przepisujemy jeszcze do tablicy
  $self->{valuesRef}->[$self->{pozycja}]=$self->{value};
#  $self->{value}=$self->{valuesRef}->[$self->{pozycja}];
#  $self->{keysValsHashRef}->{$self->{key}}=$self->{value};
  my $komentarz=$self->{commentsRef}->[$self->{pozycja}];
  $komentarz="" if(!defined($komentarz));
  $komentarz =~ s/^\*\#//mg;
  $komentarz =~ s/\#//mg;
  $komentarz =~ s/^\s*\n//mg;
  $self->{description}=$komentarz;
  if(defined($self->{descriptionWidget})){
    $self->{descriptionWidget}->delete('0.0','end') ;
    $self->{descriptionWidget}->insert('0.0',$komentarz) ;
  }
  $self->{alwaysShowing}=$self->{visiblesRef}->[$self->{pozycja}];
};

###########################################################################
#
# Aktualizacja konfigurowanej tablicy na podstawie wpisanych wartoci
#

sub get{
  my $self = shift;
  $self->{keysValsHashRef}->{$self->{key}}=$self->{value};
#  $self->{value}=$self->{keysValsHashRef}->{$self->{key}};
  $self->{keysRef}->[$self->{pozycja}]=$self->{key};
  $self->{valuesRef}->[$self->{pozycja}]=$self->{value};
  $self->{visiblesRef}->[$self->{pozycja}]=$self->{alwaysShowing};
};

###########################################################################
#
# Przypisanie wartoci na podstawie podanego hasza
#

sub setValues{
  my ($self,$hashRef) = @_;

  my @keys=keys(%$hashRef);

  foreach my $key (@keys){
    if(defined($self->{posRef}->{$key})){
      my $pos=$self->{posRef}->{$key};
      $self->{valuesRef}->[$pos]=$hashRef->{$key};
    }else{
      print "setValue: Zly klucz: $key\n" if($key ne 'noKey');
    }
  }
};

###########################################################################
#
# Zapisanie wartoci do podanego hasza
#

sub getValues{
  my ($self,$hashRef) = @_;

  for(my $n=0;$n<@{$self->{keysRef}};++$n){
    my $key=$self->{keysRef}->[$n];
    $hashRef->{$key}=$self->{valuesRef}->[$n] if($key ne 'noKey');
  }
};

###########################################################################
#
# Ustala czy parametr grnego okna jest odstpny do modyfikacji
# (niedostpny, gdy wywietla si w dolnym oknie)
#

sub setVisibility{
  my $self = shift;
  for(my $i=0;$i<@{$self->{keysRef}};++$i){
    if(defined($self->{paramEntry}->[$i])){
      if($i==$self->{pozycja}){
        $self->{paramEntry}->[$i]->configure(-state => 'disabled', -background => 'gray');
      }else{
        $self->{paramEntry}->[$i]->configure(-state => 'normal', -background => 'white');
      }
    }
  }
}

###########################################################################
#
# Wywietl wczeniejszy parametr
#

sub go_left{
  my $self = shift;
  $self->get();
  do{
    $self->{pozycja}=$self->{ostatniaPozycja} if(--$self->{pozycja} <0) ;
  }until(
     $self->{keysRef}->[$self->{pozycja}] ne 'noKey' &&
     ($self->{visiblesRef}->[$self->{pozycja}] || $self->{showAll})
  );
  $self->set();
  $self->setVisibility();
};

###########################################################################
#
# Wywietl pniejszy parametr
#

sub go_right{
  my $self = shift;
  $self->get();
  do{
    $self->{pozycja}=0 if(++$self->{pozycja} > $self->{ostatniaPozycja}) ;
  }until(
     $self->{keysRef}->[$self->{pozycja}] ne 'noKey' &&
     ($self->{visiblesRef}->[$self->{pozycja}] || $self->{showAll})
  );
  $self->set();
  $self->setVisibility();
};

###########################################################################
#
# Przeczytaj plik konfiguracyjny i domylny plik konfiguracyjny
#

sub load{
  my $self = shift;

  my ($text,$extraText);
  if(-r $self->{file}){
    open(IN,"<$self->{file}") || die("Failed to open file $self->{file}\n");
    my $size = -s $self->{file};
    read(IN,$text,$size);
    close(IN);
  }


  # Gwiazdka midzy haszami oznacza conf sformatowany wedug nowego formatu
  # Jeli jej nie byo, bd nie byo w ogle pliku conf, czytamy plik wzorcowy
  if(!(-r $self->{file}) || $text !~ /\#\*\#/ms){
#  if($text !~ /\#\*\#/ms){
    $extraText=$text;
    open(IN,"<$self->{iniFile}") || die("Failed to open file $self->{iniFile}\n");
    my $size = -s $self->{iniFile};
    read(IN,$text,$size);
    close(IN);
  }

  # Rozpoczynamy od przeczytania dobrze sformatowanego conf
  # Najpierw dzielimy przeczytany tekst, gdy po nowej linii s trzy hasze
  # Czytanie jest skomplikowane, bo potrzebujemy zapamieta pozycje
  # danych w pliku, by potem zostay dobrze odtworzone przy zapisywaniu
  #
  my @chunks=split(/^\n+\s*###/m,$text);
  my $pos=0;
  my $visPos=0;
  foreach my $chunk (@chunks){
#    if($chunk =~ /^(.*)\n\s*(\w+)\s*=([^\n]*)$/ms){
#print "Trying ===$chunk===\n";
# Dziki poniszej modyfikacji wymagajcej * lub # na pocztku mona uywa znaku innego ni # lub * do oznaczenia parametrw
# obecnych w pliku konfiguracyjnym, ale nie konfigurowalnych przez konfigurator, na przykad znaku !
    if($chunk =~ /^([\*\#].*)\n\s*(\w+)\s*=([^\n]*)$/s){
      my ($comment,$key,$value)=($1,$2,$3);
#print "Got ===$comment===\n";
      $self->{keysRef}->[$pos]=$key;
      $self->{posRef}->{$key}=$pos;
      $self->{valuesRef}->[$pos]=$value;
      $self->{commentsRef}->[$pos]=$comment;
      # Zawsze widoczny jest parametr, w ktrego komentarzu jest ramka  #(oraz pierwszy)
#      $self->{visiblesRef}->[$pos]=($comment =~ /^\*/ || $visPos == 0);
      $self->{visiblesRef}->[$pos]=($comment =~ /^\*/);
      $visPos++;
    }else{
      $self->{keysRef}->[$pos]='noKey';
      $self->{valuesRef}->[$pos]='XXX';
      $self->{commentsRef}->[$pos]=$chunk;
      $self->{visiblesRef}->[$pos]=0;
    }
    $pos++;
  }

  # Jeli plik konfiguracyjny by starego typu, uzupeniamy parametry o
  # wartoci w tym pliku
  if($extraText){
    # Dzielimy extra text na linie
    my @chunks=split(/\n/m,$extraText);
    foreach my $chunk (@chunks){
      if($chunk =~ /^\s*(\w+)\s*=([^\n]*)$/ms){
        my ($key,$value)=($1,$2,$3);
        if(my $pos=$self->{posRef}->{$key}){
          $self->{keysRef}->[$pos]=$key;
          $self->{valuesRef}->[$pos]=$value;
        }else{
          print "Niedopuszczalny klucz: $key\n";
        }
      }
    }
  }

  $self->reset();
  $self->getValues($self->{keysValsHashRef});
}

###########################################################################
#
# Zapisz now posta pliku konfiguracyjnego
#

sub save{
  my $self = shift;

  $self->setValues($self->{keysValsHashRef});
  open(OUT,">$self->{file}") || die("Failed to open file $self->{file}\n");
  for (my $pos=0;$pos<=$#{@{$self->{keysRef}}};$pos++){
    my $key=$self->{keysRef}->[$pos];
    my $comment=$self->{commentsRef}->[$pos];
    # Parametry, ktre maj by zawsze widoczne oznaczamy gwiazdk w czwartej kolumnie
    # linii zawierajcej same znaki # (hasz)
    if($self->{visiblesRef}->[$pos]){
      $comment =~ s/^([\*\#])/\*/m;
    }else{
      $comment =~ s/^([\*\#])/\#/m;
    }
    my $value=$self->{keysValsHashRef}->{$key};
    print OUT "$comment\n";
    print OUT "$key=$value\n" if($key ne 'noKey');
    print OUT "\n###";
  }
  close(OUT);
};

###########################################################################

1;