###############################
#
# Klasa konfigurator
#
# (c) Marian Mrozek 2002
#
#  Klasa umoliwia interaktywn aktualizacj
#  plikw konfiguracyjnych zapisanych w formacie
#  klucz=warto
#  wykorzystujcych znak # do opisu komentarzy

package konfigurator;

use Tk;
use strict;
require Tk::ROText;

################################
#
# Konstruktor
#

sub new{
  my $class = shift;
  my $self = {};
  bless($self,$class);
  (
    $self->{ramka},$self->{file},$self->{iniFile}
  ) = @_;

  $self->load();

  return $self;
}


###########################################################################
#
# Tworzy gwne okno konfiguratora i wypenia je pocztkow treci
#

sub configWindow{
  my $self = shift;

  # Tworzymy potrzebne widgety
  my $f1 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'both', -expand => 1);
  my $f2 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'x');
  my $f3 = $self->{ramka}->Frame->pack(-side => 'top', -fill => 'x');

  $self->{descriptionWidget}=$f1->Scrolled('ROText',-scrollbars => 'osoe', -width => 80, -height => 5, -relief => 'ridge', -background => 'gray')->
    pack(-side => 'left', -anchor => 'e', -fill => 'both', -expand => 1);

  $f2->Label(-textvariable => \$self->{key}, -anchor => 'c')->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f2->Entry(-textvariable => \$self->{value})->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f2->Checkbutton(-text => 'poka zawsze',-variable => \$self->{alwaysShowing}, -anchor => 'c')->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f3->Button(-text => "Poprzedni parametr", -command => ['go_left' => $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f3->Button(-text => "Nastpny parametr", -command => ['go_right' => $self] )->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  $f3->Button(-text => "Zapisz", -command => ['done' => $self] )->
    pack(-side => 'left', -fill => 'x', -expand => 1);

  $self->{pokazWszystkie}=$f3->Checkbutton(-text => 'poka wszystkie', -variable => \$self->{showAll},  -anchor => 'c', -relief => 'raised')->
    pack(-side => 'left', -anchor => 'e', -fill => 'x', -expand => 1);

  # I przechodzimy do wywietlenia pierwszego parametru
  $self->go_right();
}

################################
#
# inicjalizacja
#

sub init{
  my $self = shift;

  $self->{keysRef}=[];
  $self->{valuesRef}=[];
  $self->{commentsRef}=[];
  $self->{visiblesRef}=[];
}


###########################################################################
#
# ustawienie pocztkowe
#

sub reset{
  my $self = shift;
  $self->{ostatniaPozycja}=$#{@{$self->{keysRef}}};
  $self->{pozycja}=0;
  $self->{showAll}=0;
  $self->set();
}


###########################################################################
#
# obsuga przycisku 'Zapisz'
#

sub done{
  my $self = shift;
  $self->get();
  $self->save();
}

###########################################################################
#
# Ustawienie aktualnych wywietlanych wartoci na podstawie kongigurowanej
# tablicy
#

sub set{
  my $self = shift;
  $self->{key}=$self->{keysRef}->[$self->{pozycja}];
  $self->{value}=$self->{valuesRef}->[$self->{pozycja}];
  my $komentarz=$self->{commentsRef}->[$self->{pozycja}];
  $komentarz="" if(!defined($komentarz));
  $komentarz =~ s/^\*\#//mg;
  $komentarz =~ s/\#//mg;
  $komentarz =~ s/^\s*\n//mg;
  $self->{description}=$komentarz;
  if(defined($self->{descriptionWidget})){
    $self->{descriptionWidget}->delete('0.0','end') ;
    $self->{descriptionWidget}->insert('0.0',$komentarz) ;
  }
  $self->{alwaysShowing}=$self->{visiblesRef}->[$self->{pozycja}];
};

###########################################################################
#
# Aktualizacja konfigurowanej tablicy na podstawie wpisanych wartoci
#

sub get{
  my $self = shift;
  $self->{keysRef}->[$self->{pozycja}]=$self->{key};
  $self->{valuesRef}->[$self->{pozycja}]=$self->{value};
  $self->{visiblesRef}->[$self->{pozycja}]=$self->{alwaysShowing};
};

###########################################################################
#
# Wywietl wczeniejszy parametr
#

sub go_left{
  my $self = shift;
  $self->get();
  do{
    $self->{pozycja}=$self->{ostatniaPozycja} if(--$self->{pozycja} <0) ;
  }until(
     $self->{keysRef}->[$self->{pozycja}] ne 'noKey' &&
     ($self->{visiblesRef}->[$self->{pozycja}] || $self->{showAll})
  );
  $self->set();
};

###########################################################################
#
# Wywietl pniejszy parametr
#

sub go_right{
  my $self = shift;
  $self->get();
  do{
    $self->{pozycja}=0 if(++$self->{pozycja} > $self->{ostatniaPozycja}) ;
  }until(
     $self->{keysRef}->[$self->{pozycja}] ne 'noKey' &&
     ($self->{visiblesRef}->[$self->{pozycja}] || $self->{showAll})
  );
  $self->set();
};

###########################################################################
#
# Przeczytaj plik konfiguracyjny i zapamitaj w tablicach
#

sub load{
  my $self = shift;

  $self->init();
  my $file=(-r $self->{file} ? $self->{file} : $self->{iniFile});
  open(IN,"<$file") || die("Failed to open file $file\n");
  my $size = -s $file;
  my $text;
  read(IN,$text,$size);
  close(IN);

  # Dzielimy przeczytany tekst, gdy po nowej linii s trzy hasze
  my @chunks=split(/^\n+\s*###/m,$text);
  my $pos=0;
  foreach my $chunk (@chunks){
    if($chunk =~ /^(.*)\n\s*(\w+)\s*=([^\n]*)$/ms){
      my ($comment,$key,$value)=($1,$2,$3);
      $self->{keysRef}->[$pos]=$key;
      $self->{valuesRef}->[$pos]=$value;
      $self->{commentsRef}->[$pos]=$comment;
      $self->{visiblesRef}->[$pos]=($comment =~ /^\*/);
    }else{
      $self->{keysRef}->[$pos]='noKey';
      $self->{valuesRef}->[$pos]='XXX';
      $self->{commentsRef}->[$pos]=$chunk;
      $self->{visiblesRef}->[$pos]=0;
    }
    $pos++;
  }
  $self->reset();
};

###########################################################################
#
# Zapisz now posta pliku konfiguracyjnego
#

sub save{
  my $self = shift;

  open(OUT,">$self->{file}") || die("Failed to open file $self->{file}\n");
  for (my $pos=0;$pos<=$#{@{$self->{keysRef}}};$pos++){
    my $key=$self->{keysRef}->[$pos];
    my $comment=$self->{commentsRef}->[$pos];
    # Parametry, ktre maj by zawsze widoczne oznaczamy gwiazdk w czwartej kolumnie
    # linii zawierajcej same znaki # (hasz)
    if($self->{visiblesRef}->[$pos]){
      $comment =~ s/^([\*\#])/\*/m;
    }else{
      $comment =~ s/^([\*\#])/\#/m;
    }
    print OUT "$comment\n";
    print OUT "$key=$self->{valuesRef}->[$pos]\n" if($key ne 'noKey');
    print OUT "\n###";
  }
#  print OUT "########################################################################\n";
  close(OUT);
};

###########################################################################

1;