###############################
#
# modu programw pomocniczych
# (c) Marian Mrozek, Krakw - Nowy Scz 1996-2001
#  wersja 1.11 (2001.01.08)

package utilities;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(fileModificationTime getSysop adjustfn copy move eject_blanks eject_right_blanks eject_quotations remove_quotations eject_blanks_in_array dirSlash pwd menuSelection readConfiguration forgetConfiguration redoOptionsFromParametr fill_to_left fill_to_right fill_with_zeros date_time randomPerm confirmContinuation addToHashOfArrays arrayElementTest keysSortedByValues makeDir);


#############################################################
#
# Funkcja zwraca czas ostatniej modyfikacji pliku o podanej nazwie
#
#

sub fileModificationTime{
  my $file=$_[0];
  if(-r $file){
    my $t=(stat($file))[9];
    if ($t eq ""){
      $t=`filemodt $file`;
      $t=chomp($t);
    }
    return $t;
  }else{
    return 'none';
  }
}

#########################################################################
#
# Funkcja zwraca WIN32 lub UNIX w zalenoci od systemu operacyjnego
#
#
sub getSysop{
  if(!defined($main::sysop)){
    $main::sysop=(`cd` =~ /:/ ? 'WIN32' : 'UNIX');
  }
  return $main::sysop;
}

############################################################
#
# Funkcja dostosowuje wygld cieki do systemu operacyjnego
#
sub adjustfn{
  my ($arg) = @_;
  if(getSysop() eq "WIN32"){
    $arg =~ s|/|\\|g;
  }else{
    $arg =~ s|\\|/|g;
  }
  return $arg;
}

#############################################################
#
# Funkcja kopiuje plik stosownie do systemu operacyjnego
#
sub copy{
  my $sourcefile=adjustfn($_[0]);
  my $destfile=adjustfn($_[1]);
  my $sysop=getSysop();
  if($sysop eq "WIN32"){
    $sourcefile =~ s/\//\\/g;
    $destfile =~ s/\//\\/g;
    my $copy="copy \"$sourcefile\"  \"$destfile\"";
    my $res=`$copy`;
  }elsif($sysop eq "UNIX"){
    $sourcefile =~ s/\\/\//g;
    $destfile =~ s/\\/\//g;
    my $res=`cp \"$sourcefile\"  \"$destfile\"`; #"
  }else{
    die("utilities::copy: undetermined operating system\n");
  }
  return $res;
}

#############################################################
#
# Funkcja przenosi plik stosownie do systemu operacyjnego
#
sub move{
  my $sourcefile=adjustfn($_[0]);
  my $destfile=adjustfn($_[1]);
  my $sysop=getSysop();
  if($sysop eq "WIN32"){
    $sourcefile =~ s/\//\\/g;
    $destfile =~ s/\//\\/g;
    my $move="move \"$sourcefile\"  \"$destfile\"";
# print "Doing $move\n";
    my $res=`$move`;
  }elsif($sysop eq "UNIX"){
    $sourcefile =~ s/\\/\//g;
    $destfile =~ s/\\/\//g;
    my $res=`mv \"$sourcefile\"  \"$destfile\"`; #"
  }else{
    die("utilities::move: undetermined operating system\n");
  }
  return $res;
}

#########################################################################
#
# Funkcja zwraca argument pozbawiony zewntrznych biaych znakw
#
sub eject_blanks{
  my ($arg)=@_;
  if ($arg =~ /^\s*(\S.*\S)\s*$/ms){
    $arg=$1;
  }elsif($arg =~ /^\s*(\S)\s*$/ms){
    $arg=$1;
  }else{
    $arg='';
  }
  return $arg;
}

#########################################################################
#
# Funkcja zwraca argument pozbawiony zewntrznych biaych znakw
#
sub eject_right_blanks{
  my ($arg)=@_;
  if ($arg =~ /^(\s*\S.*\S)\s*$/ms){
    $arg=$1;
  }elsif($arg =~ /(^\s*\S)\s*$/ms){
    $arg=$1;
  }else{
    $arg='';
  }
  return $arg;
}

#########################################################################
#
# Funkcja zwraca argument pozbawiony zewntrznych biaych znakw i cudzysoww
#
sub eject_quotations{
  my ($arg)=@_;
  if ($arg =~ /^\s*\"\s*(\S.*\S?|\S|)\s*\"\s*$/ms){ #"
    $arg=$1;
  }
  return $arg;
}

#########################################################################
#
# Funkcja zwraca argument pozbawiony cudzysoww
#
sub remove_quotations{
  my ($arg)=@_;
  $arg =~ s/\"//g;
  return $arg;
}

##########################################
# Funkcja odrzuca zewntrzne blanki we wszystkich elementach podanej tablicy
#
sub eject_blanks_in_array{
  my @arg;
  my $i;
  for ($i=0;$i<=$#_;$i++){
     $arg[$i]=eject_blanks($_[$i]);
  }
  return @arg;
}

#########################################################################
#
# Funkcja zwraca podany argument uzupeniony o kocowy slash
# jeli argument jest niepusty i slasha na kocu nie ma
#
sub dirSlash{
  my ($arg)=@_;
  $arg .= '/' if($arg =~ /[^\/\s]$/);
  return $arg;
}

#########################################################################
#
# Funkcja zwraca katalog biecy
#
sub pwd{
  if(getSysop() eq 'WIN32'){
    return eject_blanks(`cd`);
  }else{
    return eject_blanks(`pwd`);
  }
}

#########################################################################
#
# Funkcja menuSelection wyswietla menu z elementami podanymi w tablicy
# i zwraca numer wybranej pozycji.
#
#     ----------- jeszcze nie uywana --------------
#
sub menuSelection{
  my @choices=@_;
  my ($choice,$selection,$i)=('',0,0);
  while($selection<1 || $selection > $#choices+1){
    $i=0;
    foreach $choice (@choices){
      $i++;
      print "$i - $choice\n";
    }
    print "---\nSelect your choice and press ENTER.\n";
    $selection = <>;
    chop $selection;
  }
  return $selection;
}

#########################################################################
#
# Funkcja przeglda katalog podany jako argument $dir i jego nadkatalogi
# do wysokoci podanej przez argument $depth w poszukiwaniu plikw
# konfiguracyjnych o podanej nazwie lub domylnie o nazwie conf.
# Pliki te s czytane. Linie puste i rozpoczynajce
# si od znaku # s pomijane
# Dla pozostaych linii zakadana jest skadnia klucz=wartosc
# Pary tej postaci wstawiane sa do hasza podanego przez referencj $parametr
# o ile pola o znalezionym kluczu jeszcze w tym haszu nie byo.
# Jeli parametru tego nie podano, generowana jest referencja do pustego hasza.
# Tak czy owak referencja do hasza jest zwracana jako warto funkcji.
# W przypadku gdy klucz pojawia si w wicej ni w jednym pliku konfiguracyjnym,
# uywana jest warto znaleziona najwczeniej, t.j. w pliku conf najbliszym katalogu
# podanego w argumencie $dir
# Jeli argument $dir zawiera '.', przegldanie rozpoczyna si od katalogu biecego.
#
sub readConfiguration{
  my ($dir,$depth,$parametr,$confFile,$reRead)=@_;

  $confFile = 'conf' if(!$confFile);
  $parametr = {} if(!$parametr);
  $dir=pwd() if($dir eq '.');
  $dir =~ s/\\/\//g;
  my $sysop=$parametr->{sysop}=(`cd` =~ /:/ ? 'WIN32' : 'UNIX');
  my $fs=$parametr->{fs}=($sysop eq 'UNIX' ? '/':'\\');
  my $confDir="$dir/";
  my ($i,$confFound,$baseDirPath)=(0,0,'');

  CONFDIRLOOP:
  for($i=0;$i<$depth;$i++){
# print "readConfiguration, trying $i($confDir$confFile)\n";
   if(open(CONF,"<$confDir$confFile")){
# print "reading....par=$parametr\n";
      $confFound++;
      $parametr->{confDir}=$confDir if($confFound == 1);
      while(<CONF>){
        if( !($_ =~ /^\s*\#/ || $_ =~ /^\s*$/) ){
          s/\n//g;
          if($_ =~ /^([^=]+)=(.*)$/){
            my ($nazwa,$wartosc) = eject_blanks_in_array($1,$2);
            # Excel czasem produkuje pola w cudzysowach. Trzeba je zdj.
            if($wartosc =~ /^\s*\"?(.*?)\"?\s*$/){ #"
              $wartosc=$1;
            }
            # Parametry wpisujemy tylko gdy nie byy jeszcze ustawione,
            # chyba, e zadano ich odnowienia (reRead)
            if(!defined($parametr->{$nazwa}) || $reRead ){
              $parametr->{$nazwa} = $wartosc;
#              print "Read $nazwa=$wartosc\n" if($main::opt_v);
            }
          }
        }
      }
      close(CONF);
    }
    if($confDir =~ /^(.*\/)([^\/]+)\/$/){
      $confDir=$1;
    }else{
#      die("Failed to decompose base directory=$confDir\n");
       last CONFDIRLOOP;
    }
    $baseDirPath.='..$fs';
  }
  die("Failed to find configuration file named $confFile in $confDir\n") if (!$confFound);
  return ($parametr);
}

#########################################################################
#
# Poszukuje pliku w podkatalogach.
# Jeli go znajdzie, zwraca ciek dostpu
# Nieprzetestowane
#
  sub serachDownFile{
  my ($dir,$depth,$file)=@_;

  $dir=pwd() if($dir eq '.');
  $dir =~ s/\\/\//g;
  my $fileDir="$dir/";
  my $fileWithPath="$dir/$file";

  CONFDIRLOOP:
  for($i=0;$i<$depth;$i++){
    if(-r $fileWithPath){
      last CONFDIRLOOP;
    }
    if($fileDir =~ /^(.*\/)([^\/]+)\/$/){
      $fileDir=$1;
      $fileWithPath="$fileDir$file";
    }else{
       $fileWithPath='';
       last CONFDIRLOOP;
    }
  }
  return $fileWithPath;
}

#########################################################################
#
# Funkcja przeglda hasz podany w parametr i jesli znajdzie klucz postaci opt_*
# przypisuje warto klucza zmiennej $main::opt_*
#
#
sub redoOptionsFromParametr{
  my ($parametr)=@_;
  my $key;
  my @k=keys(%$parametr);
  foreach $key(@k){
    if($key =~ /^(opt_\w)$/){
      my $var="main::$1";
      ${$var}=$parametr->{$key};
    }
  }
}

##########################################
#
#  Uzupelnianie spacjami z prawej strony do poprawnego wydruku
#
sub fill_to_right{
  my ($word,$size)=@_;
  my $l=length($word);
  if($l>=$size){
    return $word;
  }else{
    for($j=0;$j<$size-$l;$j++){
      $word.=' ';
    }
    return $word;
  }
}

##########################################
#
#  Uzupelnianie spacjami z lewej strony do poprawnego wydruku
#
sub fill_to_left{
  my ($word,$size,$filler)=@_;
  $filler=' ' if(!defined($filler) || $filler eq '');
  my $l=length($word);
  if($l>=$size){
    return $word;
  }else{
    for($j=0;$j<$size-$l;$j++){
      $word=$filler.$word;
    }
    return $word;
  }
}

##########################################
#
#  Uzupelnianie zermi z lewej strony do poprawnego wydruku
#
sub fill_with_zeros{
  my ($word,$size)=@_;
  my $l=length($word);
  if($l>=$size){
    return $word;
  }else{
    for($j=0;$j<$size-$l;$j++){
      $word='0'.$word;
    }
    return $word;
  }
}

############################### date #########################
#
#  Zwraca dat i czas
#
sub date_time{
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydat,$isdst)=localtime();
  $year+=1900;
  @weekday=(Sun,Mon,Tue,Wed,Thu,Fri,Sat);
  @month=(Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec);

  return "$weekday[$wday], $year/$month[$mon]/$mday, $hour:$min:$sec";
}

############################### randomPerm #########################
#
#  Zwraca losow permutacj elementw podanej tablicy
#
sub randomPerm{
  my @arg=@_;
  my @res=();
  my $n=$#arg+1;
  my $i;
  my %hash=();
  for ($i=0;$i<$n;$i++){
    $hash{$arg[$i]}=$arg[$i];
  }
  for ($i=$n;$i>0;$i--){
    @arg=keys(%hash);
    $res[$n-$i]=delete($hash{$arg[int($i*rand(1))]});
  }
  return @res;
}

############################### confirm #########################
#
#  Prosi o potwierdzenie
#
sub confirmContinuation{
  my ($question)=@_;
  print "---\n$question\nCzy kontynuowac? Odpowiedz tak lub nie.\n";
  $inp = <>; chop $inp;
  if($inp ne "Tak" && $inp ne "tak" && $inp ne "TAK"){die "Brak akceptacji. Rezygnuje.\n";};
}

##########################################
# dopisuje element do tablicy zadanej kluczem
# w Hashu tablic
#
sub addToHashOfArrays{
  my ($hashRef,$hashKey,$element)=@_;
  if(!defined($hashRef->{$hashKey})){
     $hashRef->{$hashKey}=[];
  }
  push(@{$hashRef->{$hashKey}},$element);
}

#################################################
# sprawdza czy podany element jest elementem podanej tablicy
#
sub arrayElementTest{
  my ($element,$arrayRef)=@_;
  my $i;
  my $result=0;
  foreach $i (@$arrayRef){
    $result =1 if($i eq $element);
  }
  return $result;
}

#################################################
# zwraca tablic kluczy hasza posortowan wedug wartoci
#
sub keysSortedByValues{
  my ($hashRef)=@_;
  my %inverse;
  my @result;
  while(my($key,$value)=each(%$hashRef)){
    if(!defined($inverse{$value})){
      $inverse{$value}=$key;
    }else{
      print "Value $value appears more than once. Key $key will be lost\n";
    }
  }
  my @values=keys(%inverse);
  my @sortedValues=sort(@values);
  for (my $i=0;$i<=$#sortedValues;$i++){
    $result[$i]=$inverse{$sortedValues[$i]};
  }
  return @result;
}

#################################################
# tworzy folder w oparciu o ciek wraz z potrzebnymi
# folderami po drodze
#
sub makeDir{
  my ($path,$localization)=@_;
  $localization="." if(!$localization);
  my @dirs=split('/',$path);
  my $newpath=$localization;
  foreach my $dir (@dirs){
    $newpath.="/$dir";
    mkdir($newpath,0770) if(!(-d $newpath) && !$main::opt_t);
  }
}

#########################################################################

